# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2018 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""fmf_tools.py: Module to manipulate FMF."""

import os
import fmf.utils
import copy
import stqe
from fmf import Tree
from random import shuffle
from libsan.host.cmdline import run
from stqe.host.persistent_vars import read_env, read_var, get_persistent_files_dir, get_persistent_vars_file_name

_INSTALL_SCRIPT = """#!/usr/bin/python
from libsan.host.linux import install_package

install_package(pack=, check=False)
"""


class TestNotFound(Exception):
    """Raise when test case could not be found"""
    pass


def remove_nones(kwargs):
    # removes keys from kwargs dict that are Nones
    # returns the same dict without those keys
    return {k: v for k, v in kwargs.items() if v is not None}


def get_stqe_path():
    stqe_path = stqe.__file__
    return stqe_path.replace('__init__.py', '')


def get_dir_from_name(name):
    if not os.path.isdir(get_stqe_path() + "/tests/" + name):
        return "/".join(name.split("/")[:-1])
    return name


def get_tree():
    """
    Returns FMF Tree object initialized in stqe tests root directory.
    :return:
    :rtype Tree
    :return: FMF Tree object with root in stqe tests path
    """
    path = get_stqe_path() + "/tests/"
    return Tree(path)


def get_tests_path(name):
    return get_stqe_path() + "/tests/" + get_dir_from_name(name)


def get_test_name(tree):
    test_path = tree.get('test')
    if test_path.startswith("/"):
        return test_path
    remove, test_path = correct_path(test_path)
    return "/".join(tree.name.split("/")[1:-(1 + remove)]) + "/" + test_path


def correct_path(path):
    # Sometimes there is '../$path' in path, we need get number of these and remove them from the path
    remove = 0
    while "../" in path:
        path = "/".join(path.split("/")[1:])
        remove += 1
    return remove, path


def remove_redundant(leaf):
    for att in ['tester', 'description', 'active', 'requires_install', 'requires_cleanup', 'requires_setup',
                'component', 'storage']:
        try:
            del leaf[att]
        except KeyError:
            pass
    return leaf


def get_tests(tree, name=None, filter=None, os_env_asterix=None, bz=None):
    filter = filter or list("")

    config = {
        "install": [],
        "tests": [],
        "setups": [],
        "test_cleanups": {}
    }

    for i in tree.climb():
        leaf = i.get()

        # Ignore yaml aliases
        if "_alias" in i.name:
            continue

        # do not run deactivated tests
        if bz is None and 'active' in leaf and not leaf['active']:
            continue

        # no file associated with this metadata, skipping
        if 'test' not in leaf:
            continue
        # replace test value with path from stqe/tests
        leaf['test'] = get_test_name(i)

        # TODO: Do not run tests with leaf['bz'] unless asked for (bz is set)
        # Run only tests with given bz number
        if bz is not None and 'setup' not in leaf and \
                ('bz' not in leaf or not any([True for l in leaf['bz'] if l == bz])):
            # print "DEBUG: %s" % leaf['test']
            continue

        test_name = i.name.split("/")
        if len(test_name) > 1:
            leaf['name'] = "/".join(test_name[1:])
        else:
            leaf['name'] = i.name

        # do not run setups / cleanups, they are ran by specifying 'requires_setup' / 'requires_cleanup' in test meta
        # add them to "setups" so we can locate them later without running though the tree again
        if 'setup' in leaf and leaf['setup']:
            config["setups"].append(dict(leaf))

        config["test_cleanups"][leaf['name']] = dict(leaf)
        if 'cleanup' in leaf and leaf['cleanup'] or 'setup' in leaf and leaf['setup']:
            continue

        # skip test that doesn't contain string specified with 'name' parameter
        if name is not None and name not in i.show(brief=True):
            continue

        # apply filter
        if not all([fmf_filter(filter=f, node=dict(leaf)) for f in filter]):
            continue

        config, leaf = insert_install(config, leaf)

        if 'requires_setup' not in leaf:
            # this test does not require setup
            is_in_tests = _is_test_in_tests(leaf['name'], config['tests'])
            if isinstance(is_in_tests, bool) and not is_in_tests:
                config['tests'].append(dict(leaf))
            elif isinstance(is_in_tests, int):
                # replace test if it is greater tier
                config['tests'][is_in_tests] = leaf
            test_position = _get_test_position(leaf['name'], config['tests'])
            try:
                if test_position is not None and leaf['tier'] > config['tests'][test_position]['tier']:
                    # replacing tier with higher one
                    config['tests'][test_position] = leaf
            except KeyError:
                pass
            if not _is_test_in_tests(leaf['name'], config['tests']):
                # test is not in yet
                config['tests'].append(dict(leaf))
        else:
            requires_setup = replace_require_setup(leaf, os_env_asterix)
            config = insert_test(config, requires_setup, leaf)
    return config


def insert_install(tests, leaf):
    # write install test file for each package and adds them to the beginning of tests
    if 'requires_install' not in leaf:
        return tests, leaf
    requires_install = leaf.pop('requires_install')
    if not isinstance(requires_install, list):
        requires_install = [requires_install]
    for to_install in requires_install:
        inst_file = "install_%s.py" % to_install
        if not os.path.isfile(get_stqe_path() + "/tests/" + inst_file):
            with open(get_stqe_path() + "/tests/" + inst_file, 'w') as f:
                script = _INSTALL_SCRIPT.split(',')
                script.insert(1, "'%s'," % to_install)
                # write the inst file itself
                f.writelines("".join(script))
                # write inst file to config to be ran
            # make it executable so it can be ran as a test case
            run("chmod +x %s" % (get_stqe_path() + "/tests/" + inst_file))
        inst = {'test': inst_file, 'name': inst_file, 'install': True}
        inst["test_type"] = "install"
        if inst not in tests['install']:
            tests['install'].insert(0, inst)
    return tests, leaf


def _get_test_position(test, tests):
    for i, t in enumerate(tests):
        if test == t['name']:
            return i
    return None


def _is_test_in_tests(test, tests):
    for i, t in enumerate(tests):
        if test == t['name']:
            return True
    return False


def replace_require_setup(leaf, os_env=None):
    require_setup = leaf.pop('requires_setup')
    # check if the setup requirement needs to be altered (contains "*")
    if not isinstance(require_setup, list):
        require_setup = [require_setup]
    req = ""
    for requirement in require_setup:
        if "*" in requirement:
            if os_env is None:
                print("FAIL: Set '%s' env to specify setup type for tests." % os_env)
                exit(1)
            env = os_env.split()
            # replace '*' with the correct device type and adds the rest setup functions
            req = ""
            for i, x in enumerate(env):
                if i == 0:
                    req += x.join(requirement.split("*"))
                else:
                    req += " " + x
        else:
            req += requirement
        req += " "
    return req


def insert_test(dictionary, name, test):
    name = [x.strip() for x in name.split()]
    if len(name) == 1:
        this_name = name[0]
        name = None
    else:
        this_name = name.pop(0)
        name = " ".join(name)
    if this_name not in dictionary:
        dictionary[this_name] = dict()
        dictionary[this_name]['tests'] = []
    for key in dictionary:
        if isinstance(dictionary[key], dict) and name is not None and key == this_name:
            insert_test(dictionary[key], name, test)
        if name is None and key == this_name:
            test_position = _get_test_position(test['name'], dictionary[key]['tests'])
            try:
                if test_position is not None and test['tier'] > dictionary[key]['tests'][test_position]['tier']:
                    # replacing tier with higher one
                    dictionary[key]['tests'][test_position] = test
            except KeyError:
                pass
            if not _is_test_in_tests(test['name'], dictionary[key]['tests']):
                # test is not in yet
                dictionary[key]['tests'].append(dict(test))
    return dictionary


def fmf_filter(filter, node):
    try:
        return fmf.utils.filter(filter=filter, data=node)
    except fmf.utils.FilterError:
        # If the attribute is missing, return as not found
        return False


def filter_tree(filters, name=None, verbose=False, to_print=False):
    # returns list of tests filtered by given filter
    #         "tier:1 & test:this.py" --> all tests that run 'this.py' on tier 1
    #         "tier:1 | test:this.py" --> all tests that run 'this.py' or are on tier 1
    # verbose True: returns list of dicts of test metadata
    #         False: return list of strings of test names

    if not isinstance(filters, list):
        raise fmf.utils.FilterError("Invalid filter '%s'" % type(filters))
    for f in filters:
        if not isinstance(f, str):
            raise fmf.utils.FilterError("Invalid filter '%s'" % type(f))
    tree = get_tree()
    filtered = []
    for leaf in tree.climb():
        # skip test that doesn't contain string specified with 'name' parameter
        if name is not None and name not in leaf.show(brief=True):
            continue
        try:
            if not all([fmf_filter(filter=f, node=leaf.data) for f in filters]):
                continue
        # Handle missing attribute as if filter failed
        except fmf.utils.FilterError:
            continue
        if not verbose:
            filtered.append(leaf.show(brief=True))
        else:
            if to_print:
                filtered.append(dict(leaf.show()))
            else:
                filtered.append(dict(leaf.data))
    return filtered


def get_config(main_dictionary, dictionary=None, conf=None, sort=False):
    conf = conf or list()
    if dictionary is None:
        dictionary = main_dictionary
    if sort:
        dictionary['tests'].sort(key=lambda x: x['name'])
    else:
        shuffle(dictionary['tests'])
    for test in dictionary['tests']:
        test["test_type"] = "test"
        conf.append(remove_redundant(dict(test)))
        if "requires_cleanup" in test:
            # append all cleanups required for this test
            for test_cleanup in test["requires_cleanup"]:
                test_cleanup = get_test_cleanup(main_dictionary, test_cleanup)
                test_cleanup["test_type"] = "cleanup"
                conf.append(remove_redundant(dict(test_cleanup)))
    for key in dictionary:
        # This is not a test, just a dictionary of containing other tests/setups...
        if key in ['tests', 'install', 'setups', 'test_cleanups']:
            continue
        setup = copy.deepcopy(get_setup(main_dictionary, key))
        setup["test_type"] = "setup"
        cleanup = None
        if 'requires_cleanup' in setup:
            cleanup = setup.pop('requires_cleanup')
        conf.append(remove_redundant(dict(setup)))
        if isinstance(dictionary[key], dict):
            conf = get_config(main_dictionary, dictionary[key], conf, sort=sort)
        if cleanup is not None:
            if not isinstance(cleanup, list):
                cleanup = [cleanup]
            for c in cleanup:
                c = get_setup(main_dictionary, c)
                c["test_type"] = "cleanup"
                conf.append(remove_redundant(dict(c)))
                conf, leaf = insert_install(conf, c)
    if dictionary == main_dictionary:
        for install in main_dictionary['install']:
            conf.insert(0, remove_redundant(install))
    return conf


def get_test_cleanup(dictionary, name):
    if name in dictionary["test_cleanups"]:
        if 'requires_install' in dictionary["test_cleanups"][name]:
            insert_install(dictionary, dictionary["test_cleanups"][name])
        return dictionary["test_cleanups"][name]
    msg = "FAIL: Could not find cleanup %s!" % name + "\nPossible cleanups: \n\t%s" % \
          "\n\t".join(dictionary["test_cleanups"].keys())
    raise TestNotFound(msg)


def get_possible_setups(dictionary, name):
    return ["_".join(x['name'].split("/").pop().split("_")[:]) for x in dictionary if
            "storage" in x and x["storage"] and x['name'].split("/")[:-1] == name.split("/")[:-1]]


def get_setup(dictionary, name):
    for setup in dictionary["setups"]:
        if setup['name'] == name:
            if 'requires_install' in setup:
                insert_install(dictionary, setup)
            return setup
    msg = "FAIL: Could not find setup %s!" % name + "\nPossible setups: \n\t%s" % \
          "\n\t".join(get_possible_setups(dictionary["setups"], name))
    raise TestNotFound(msg)


def get_report(report, main, config, padding="", no_metadata=False, metadata=None):
    metadata = metadata or list(['description', 'tier'])

    def add_metadata(_leaf, _padding, _ending, _metadata):
        data = ""
        if _ending == "└── ":
            _padding += "    "
        else:
            _padding += "│   "

        _ending = "├── "
        for i, item in enumerate(_metadata):
            if i == len(_metadata) - 1:
                _ending = "└── "
            try:
                data += _padding + _ending + "%s: %s" % (item, _leaf[item]) + "\n"
            except KeyError:
                pass
        return data

    def add_name(_name, _padding):
        return _padding + _name + "\n"

    try:
        config.pop("install")
        config.pop("setups")
        config.pop("test_cleanups")
    except KeyError:
        pass

    config["tests"].sort(key=lambda x: x['name'])
    tests = config.pop("tests")
    for t, test in enumerate(tests):
        ending = "├── "
        if t + 1 == len(tests) and len(config) == 0:
            ending = "└── "
        report += add_name("T: " + test["name"], padding + ending)
        if not no_metadata:
            report += add_metadata(test, padding, ending, metadata)

    for s, setup in enumerate(config):
        name = setup
        if "*" not in setup:
            name = get_setup(main, setup)["name"]
        if s + 1 < len(config):
            report += add_name("S: " + name, padding + "├── ")
            ending = "│   "
        else:
            report += add_name("S: " + name, padding + "└── ")
            ending = "    "
        report = get_report(report, main, config[setup], padding + ending, no_metadata, metadata)

    return report


def _replace_env_args_with_persistent(args, persistent):
    """
    Automatically replaces args from environ with persistent values stored in files
    :param args: dict of env args already filled
    :param persistent: list of possible persistent vars
    :return: modified args
    """

    def _is_part_of_longer_persistent_var_name(var):
        # need to check if the persistent var is not just part of some longer persistent var name
        # checks char before and char after
        for char_position in [[0, -1], [1, 0]]:
            try:
                char = var.split(p)[char_position[0]][char_position[-1]]
                if char == "_" or char.isupper() and char.isalpha() or int(char):
                    break
            except (IndexError, ValueError):
                pass
        else:
            # is executed if the for loop exits normally (e.g. not by 'break')
            return False
        return True

    def _replace_value(var):
        # Recursively replaces the variable with persistent name in case of list
        if isinstance(var, list):
            for i, item in enumerate(var):
                var[i] = _replace_value(item)
        else:
            # The value needs to be string, as persistent var files are strings and its name should be in var
            # And since we now know it is a string, we can replace it
            if not isinstance(var, type("")) or p not in var:
                return var
            # The var is exactly what we want to replace, do it
            if p == var:
                return read_var(p)
            # Maybe persistent var name is there, but it is part of longer one, check it before replacing
            if not _is_part_of_longer_persistent_var_name(var):
                var = var.replace(p, str(read_var(p)))
        return var

    if not persistent:
        return args
    for arg in args:
        for p in persistent:
            if args[arg] is None:
                continue
            try:
                # List and Dict are nested, cannot compare this easily
                if not (isinstance(args[arg], list) or isinstance(args[arg], dict)) and p not in args[arg]:
                    continue
            except TypeError:
                continue
            if not os.path.isfile(get_persistent_files_dir() + p):
                print("FAIL: Persistent vars file '%s' does not exist!" % p)
                continue
            args[arg] = _replace_value(args[arg])
    return args


def _replace_keys(args, keys):
    """
    Some env variables are used for internal stuff, like 'name' or 'test'. This allows to replace them on the run.
    :param args: dict of values
    :param keys: dict of how to replace keys {original: new}
    :return: modifed args
    """
    for key in keys:
        if key in args:
            args[keys[key]] = args.pop(key)
    return args


def _apply_correct_type(values, keys):
    for value in values:
        try:
            values[value] = keys[value](values[value])
        except (KeyError, TypeError):
            pass
    return values


def get_env_args(args=None, persistent=None, replace_keys=None):
    """
    :param args: list of arguments
    :param persistent: list of possible persistent arguments
    :param replace_keys: dict of how to replace keys {original: new}
    :return: dict of env args
    """
    args = args or list()
    if not isinstance(args, dict):
        args = {arg: None for arg in args}
    persistent_vars = read_var(get_persistent_vars_file_name())
    persistent = persistent or persistent_vars if persistent_vars is not None else list()
    replace_keys = replace_keys or dict()
    arguments = {extra: None for extra in ["message", "expected_ret", "expected_out", "pkg", "command", "force"]
                 if extra not in args}
    arguments.update({a: None for a in args})
    for argument in arguments:
        try:
            arguments[argument] = read_env('fmf_' + argument)
        except KeyError:
            pass
    return _replace_keys(
        _apply_correct_type(
            remove_nones(
                _replace_env_args_with_persistent(
                    arguments,
                    persistent)),
            args),
        replace_keys)


def get_func_from_string(wrapper_object, command, local_functions=None):
    """
    :param wrapper_object: object that might contain method named in variable 'command'
    :param command: string with function to be found
    :param local_functions: dict of globals()
    :return: function pointer / None
    """
    local_functions = local_functions or dict()
    try:
        func = getattr(wrapper_object, command)
    except AttributeError:
        try:
            command = command.split(".")
            module = __import__(".".join(command[:-1]), globals(), locals(), level=0, fromlist=[command[-1]])
            func = getattr(module, command[-1])
        except KeyError:
            print("ERROR: Could not find function '%s' anywhere." % ".".join(command))
            return None
        except ValueError:
            # the function is is globals()
            func = local_functions[command[-1]]
    return func



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_create_success():
    errors = []

    vol_name = read_var("LSM_VOL_NAME")
    vol_size = read_var("LSM_VOL_SIZE")
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        for provisioning in [None, "DEFAULT", "THIN", "FULL"]:
            if provisioning is not None:
                provisioning = {'provisioning': provisioning}
            else:
                provisioning = {}
            ret, data = atomic_run("Creating volume %s with protocol %s." % (vol_name, config['protocol']),
                                   command=lsm.volume_create,
                                   name=vol_name,
                                   size=vol_size,
                                   pool=pool_id,
                                   return_output=True,
                                   errors=errors,
                                   **provisioning)
            vol_id = [line.split()[0].strip() for line in data.splitlines() if vol_name in line][0]
            atomic_run("Removing volume %s with protocol %s." % (vol_name, config['protocol']),
                       command=lsm.volume_delete,
                       vol=vol_id,
                       force=True,
                       errors=errors)
    return errors


def volume_create_fail():
    errors = []

    vol_name = read_var("LSM_VOL_NAME")
    vol_size = read_var("LSM_VOL_SIZE")
    pool_id = read_var("LSM_POOL_ID_VOLUME")
    pool_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail creating volume without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.volume_create},
            {'message': "Trying to fail creating volume without name with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': vol_size, 'pool': pool_id},
            {'message': "Trying to fail creating volume without size with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'name': vol_name, 'pool': pool_id},
            {'message': "Trying to fail creating volume without pool with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': vol_size, 'name': vol_name},
            {'message': "Trying to fail creating volume with wrong size '-1' with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': -1, 'name': vol_name, 'pool': pool_id},
            {'message': "Trying to fail creating volume with wrong size '0' with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': 0, 'name': vol_name, 'pool': pool_id},
            {'message': "Trying to fail creating volume with wrong size 'wrong' with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': "wrong", 'name': vol_name, 'pool': pool_id},
            {'message': "Trying to fail creating volume with pool 'NONEXISTENT' with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': vol_size, 'name': vol_name, 'pool': "NONEXISTENT"},
            {'message': "Trying to fail creating volume with pool name instead of ID with protocol %s" % config['protocol'],
             'command': lsm.volume_create, 'size': vol_size, 'name': vol_name, 'pool': pool_name},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_create_fail_name_conflict():
    errors = []

    vol_name = read_var("LSM_VOL_NAME")
    vol_size = read_var("LSM_VOL_SIZE")
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Creating volume %s again to hit NAME CONFLICT with protocol %s." % (vol_name, config['protocol']),
                   command=lsm.volume_create,
                   name=vol_name,
                   size=vol_size,
                   pool=pool_id,
                   expected_ret=4,
                   errors=errors)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_create_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_create_fail()
        errs += volume_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

#
# Bugs related:
#   BZ1305983 - thin pool with error target meta device causes kernel panic 
#
from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.dm import dm_set_target_type
from libsan.host.cmdline import run
from libsan.host.lvm import vg_create, vg_remove, pv_remove
from libsan.host.linux import sleep, install_package
import stqe.host.tc
import sys

TestObj = None

loop_dev = None

vg_name = "VG"
pool_name = "POOL"


def main():
    global TestObj

    install_package("lvm2")

    start_test()

    _clean_up()

    print("INFO: waiting 5 mins to make sure system did not crash after removing test VG")
    sleep(5 * 60)

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


def start_test():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    _clean_up()

    # create loop device of 10G
    global loop_dev
    loop_dev = create_loopdev("loop0", 10 * 1024)
    if not loop_dev:
        TestObj.tfail("Could not create loop device")
        return False

    if not vg_create(vg_name, loop_dev):
        TestObj.tfail("Could not create VG \"%s\"" % vg_name)
        return False

    print("INFO: Creating LVs")
    TestObj.tok("lvcreate --thinpool %s --zero n -L 5G --poolmetadatasize 4M %s"
                % (pool_name, vg_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n origin" % (vg_name, pool_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n other1" % (vg_name, pool_name))
    TestObj.tok("lvcreate -k n -s /dev/%s/origin -n snap1" % vg_name)

    print("INFO: Displaying LVs")
    TestObj.tok("lvs -a -o +devices")
    # going to metadata LV to error
    dm_dev = "%s-%s_tmeta" % (vg_name, pool_name)

    run("dmsetup table | grep tmeta")

    print("INFO:Going to set %s to error" % dm_dev)
    if not dm_set_target_type("%s" % dm_dev, "error"):
        TestObj.tfail("Could not set metadata LV to error")
        return False

    # list the devices again
    print("INFO: Going to display the devices now, it might to show some error")
    TestObj.tok("lvs -a -o +devices")

    run("dmsetup table | grep tmeta")

    print("INFO: rescaning for PV")
    TestObj.tok("pvscan")

    print("INFO: waiting 5 mins to make sure system did not crash")
    # BZ1305983 fails at this point
    sleep(5 * 60)

    return True


def _clean_up():
    global vg_name, loop_dev

    # make sure any failed device is removed
    run("dmsetup remove_all")
    if not vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        if not pv_remove(loop_dev):
            TestObj.tfail("Could not delete PV \"%s\"" % loop_dev)
        delete_loopdev(loop_dev)


main()



from stqe.host.persistent_vars import write_var, read_env, write_file, get_persistent_files_dir
from stqe.host.atomic_run import parse_ret, atomic_run
from libsan.host.stratis import Stratis
import random


def generate_key():
    errors = []
    keyfile = None
    key_length = None
    key = ""
    key_desc = None
    id = ""
    stratis = Stratis()

    try:
        key_desc = read_env("fmf_key_desc")
    except KeyError:
        pass
    try:
        id = "_" + str(read_env("fmf_key_id"))
        key_desc = key_desc + id
    except KeyError:
        pass
    try:
        keyfile = f"{read_env('fmf_keyfile')}{id}"
    except KeyError:
        pass
    try:
        key_length = read_env('fmf_key_length')
    except KeyError:
        pass
    try:
        key = read_env('fmf_keyring_key')
    except KeyError:
        pass

    if not key:
        for char in range(key_length):
            random_num = str(random.randint(0, 9))
            random_lower_char = chr(random.randint(97, 122))
            random_upper_char = chr(random.randint(65, 90))
            random_char = random.choice(
                [random_num, random_lower_char, random_upper_char])[0]
            key += random_char

    directory = get_persistent_files_dir()
    print(f"INFO: Using key: {key}")
    atomic_run(f"Writing key: {key} into file: {directory}{keyfile}",
               command=write_file,
               file_name=f"{keyfile}",
               value=key,
               errors=errors)

    atomic_run(f"Writing var STRATIS_KEYFILE{id}",
               command=write_var,
               var={f'STRATIS_KEYFILE{id}': f"{keyfile}"},
               errors=errors)

    atomic_run(f"Writing var STRATIS_KEYFILE_PATH{id}",
               command=write_var,
               var={f'STRATIS_KEYFILE_PATH{id}': f"{directory}{keyfile}"},
               errors=errors)

    atomic_run(f"Setting key with key_desc:{key_desc} and keyfile_path:{keyfile}.",
               command=stratis.key_set,
               keyfile_path=f"{directory}{keyfile}",
               key_desc=key_desc,
               errors=errors)

    atomic_run(f"Writing var STRATIS_KEY_DESC{id}",
               command=write_var,
               var={f'STRATIS_KEY_DESC{id}': f"{key_desc}"},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = generate_key()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_enable_success():
    errors = []

    vol_id = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Disabling volume %s with protocol %s." % (vol_id, config['protocol']),
                   command=lsm.volume_disable,
                   vol=vol_id,
                   errors=errors)

        atomic_run("Enabling volume %s with protocol %s." % (vol_id, config['protocol']),
                   command=lsm.volume_enable,
                   vol=vol_id,
                   errors=errors)

        return errors


def volume_enable_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail enabling volume without any paramethers with protocol %s"
                        % config['protocol'], 'command': lsm.volume_enable},
            {'message': "Trying to fail enabling volume with wrong volume with protocol %s"
                        % config['protocol'], 'vol': "WRONG", 'command': lsm.volume_enable}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_enable_no_state_change():
    errors = []

    vol_id = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Enabling enabled volume %s to hit NO_STATE_CHANGE with protocol %s." %
                        (vol_id, config['protocol']), 'vol': vol_id, 'command': lsm.volume_enable}
        ]
        for argument in arguments:
            atomic_run(expected_ret=4,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_enable_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_enable_fail()
        errs + volume_enable_no_state_change()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, clean_var
from os import environ


def remove_volume():
    print("INFO: Removing volume.")
    errors = []

    var_name = "LSM_" + environ["fmf_vol_id"]
    vol_id = read_var(var_name)

    config = list(yield_lsm_config())[0]
    if 'smis' in config['protocol']:
        print("smis protocol does not support ag removal on EMC. skipping.")
        return errors
    lsm = LibStorageMgmt(disable_check=True, **config)

    atomic_run("Removing volume %s" % vol_id,
               command=lsm.volume_delete,
               vol=vol_id,
               force=True,
               errors=errors)

    atomic_run("Removing var %s" % var_name,
               command=clean_var,
               var=var_name,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = remove_volume()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
from libsan.host.cmdline import run
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def _stripes():
    global TestObj, vg_name

    #
    # -i|--stripes 
    #
    TestObj.tok("lvcreate -i1 -l1 --thin %s/pool1" % vg_name)
    check_lv_expected_value(TestObj, "[pool1_tdata]", vg_name, {"stripes": "1"})

    TestObj.tok("lvcreate --stripes 2 -L4M -V2G -T %s/pool2" % vg_name)
    check_lv_expected_value(TestObj, "[pool2_tdata]", vg_name, {"stripes": "2"})

    TestObj.tok("lvcreate -i3 -L4M -V2G --thin %s/pool3" % vg_name)
    check_lv_expected_value(TestObj, "[pool3_tdata]", vg_name, {"stripes": "3"})

    TestObj.tok("lvcreate --stripes 4 -L4M -V2G -T %s/pool4 -n %s/lv2" % (vg_name, vg_name))
    check_lv_expected_value(TestObj, "[pool4_tdata]", vg_name, {"stripes": "4"})

    # Can not have more stripes than number of devices
    TestObj.tnok("lvcreate --stripes 5 -L4M -V2G -T %s/pool5 -n %s/lv3" % (vg_name, vg_name))

    TestObj.tok("lvremove -ff %s" % vg_name)


def _stripe_size():
    global TestObj, vg_name
    #
    # -I|--stripesize
    #
    # 1 strip will return stripesize == 0
    TestObj.tok("lvcreate -i1 --stripesize 64 -L4M  --thin %s/pool1" % vg_name)
    # stripe size 0 is returned like "0 ". See BZ1391117
    check_lv_expected_value(TestObj, "[pool1_tdata]", vg_name, {"stripes": "1",
                                                                "stripesize": "0 "})

    TestObj.tok("lvcreate -i2 --stripesize 128 -L4M  --thin %s/pool2" % vg_name)
    check_lv_expected_value(TestObj, "[pool2_tdata]", vg_name, {"stripes": "2",
                                                                "stripesize": "128.00k"})

    TestObj.tok("lvcreate -i3 --stripesize 256 -L4M  --thin %s/pool3" % vg_name)
    check_lv_expected_value(TestObj, "[pool3_tdata]", vg_name, {"stripes": "3",
                                                                "stripesize": "256.00k"})
    TestObj.tok("lvremove -ff %s" % vg_name)


def _pool_metadata_size():
    global TestObj, vg_name

    # -- poolmetadatasize
    # default poolmetadatasize is complex to calculate.
    # rounding to the PE size
    TestObj.tok("lvcreate -i1 -L8M -c 256 -T %s/pool1" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"lv_metadata_size": "4.00m"})

    # set poolmetadatasize, default unit is MiB
    TestObj.tok("lvcreate -i2 -L16M --poolmetadatasize 8 -T %s/pool2" % vg_name)
    check_lv_expected_value(TestObj, "pool2", vg_name, {"lv_metadata_size": "8.00m"})

    # Round up to full physical extent
    TestObj.tok("lvcreate -i3 -L16M --poolmetadatasize 2 -T %s/pool3" % vg_name)
    check_lv_expected_value(TestObj, "pool3", vg_name, {"lv_metadata_size": "4.00m"})

    # Too small, minimum is 2MiB
    TestObj.tok("lvcreate -i1 -L16M --poolmetadatasize 1 -T %s/pool4" % vg_name)
    check_lv_expected_value(TestObj, "pool4", vg_name, {"lv_metadata_size": "4.00m"})

    # too big poolmetadatasize (max is 16GB) 
    # test disabled as usually there is not enough space on server to create so big vg
    # TestObj.tok("lvcreate -i3 -L32G --poolmetadatasize 16G -T %s/pool5" % vg_name)
    # check_lv_expected_value("pool5", vg_name, {"lv_metadata_size" : "16.00g"})

    TestObj.tok("lvremove -ff %s" % vg_name)


def _chunk_size():
    global TestObj, vg_name

    # --chunksize, -c, default unit is KiB
    # too small, min is 64KiB
    TestObj.tnok("lvcreate -i2 -c 32 -l1 -T %s/pool1" % vg_name)
    # min size
    TestObj.tok("lvcreate -i1 -c 64 -l1 -T %s/pool1" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"chunksize": "64.00k"})

    # too big, max is 1GiB
    TestObj.tnok("lvcreate -i1 -c 2g -L2g -T %s/pool2" % vg_name)
    # min size
    TestObj.tok("lvcreate -i1 -c 1g -L1g -T %s/pool2" % vg_name)
    check_lv_expected_value(TestObj, "pool2", vg_name, {"chunksize": "1.00g"})

    # set some size
    TestObj.tok("lvcreate -i3 -c 512 -l1 -T %s/pool3" % vg_name)
    check_lv_expected_value(TestObj, "pool3", vg_name, {"chunksize": "512.00k"})
    TestObj.tok("lvremove -ff %s" % vg_name)


def _extents():
    global TestObj, vg_name

    # --extents -l %FREE, VG, PVS
    TestObj.tok("lvcreate -i4 --extents 10%%VG -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -i2 -l 10%%PVS -T %s/pool2" % vg_name)
    TestObj.tok("lvcreate -i3 -l 10%%VG -T %s/pool3" % vg_name)

    TestObj.tok("lvcreate -i4 -l 10%%FREE -T %s/pool4" % vg_name)
    TestObj.tok("lvcreate -i2 -l 100%%FREE -T %s/pool5" % vg_name)
    run("lvs -a %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)
    TestObj.tok("lvcreate -i3 -l 100%%VG -T %s/pool1" % vg_name)
    run("lvs -a %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)
    TestObj.tok("lvcreate -i3 -l 100%%PVS -T %s/pool1" % vg_name)
    run("lvs -a %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)
    TestObj.tnok("lvcreate -i2 -l 10%%invalid -T %s/pool1" % vg_name)

    # since RHEL-6.6 and 7.1
    # lvcreate man page: When expressed as a percentage,
    # the number is  treated as an approximate upper limit for the total number of physical extents to
    #  be allocated (including extents used by any mirrors, for example).
    # The reason for this is to better fulfill user's request - so he is not blamed there are miising extents.
    TestObj.tok("lvcreate -i3 -l 110%%FREE -T %s/pool1" % vg_name)
    run("lvs -a %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Provisioning Stripe test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=1280)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _stripes()

    _stripe_size()

    _pool_metadata_size()

    _chunk_size()

    _extents()

    return True


def _clean_up():
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2018 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""vdo.py: Module with test specific method for VDO."""

from stqe.host.atomic_run import atomic_run
from difflib import context_diff
from libsan.host.cmdline import run
import os
import stat
from stqe.host.fmf_tools import get_env_args
from stqe.host.persistent_vars import read_var
from libsan.host.vdo import VDO
from libsan.host.linux import get_memory


def restart_modify(vdo_object, vdo_name, errors):
    # sometimes VDO needs to be restarted for changes to take effect
    print("Restarting VDO for changes to take effect.")
    commands = [
        {'message': "Stopping VDO to apply modification.", 'command': vdo_object.stop,
         'expected_out': ["Stopping VDO"]},
        {'message': "Starting VDO to apply modification.", 'command': vdo_object.start,
         'expected_out': ["Starting VDO", "VDO instance", "is ready"]}
    ]
    for command in commands:
        atomic_run(name=vdo_name,
                   errors=errors,
                   expected_ret=0,
                   **command)
    return


def report_modify_difference(errors, status_old, status_new, changed_var, changed_argument):
    status_old = [x for x in status_old.splitlines() if changed_var in x]
    status_new = [x for x in status_new.splitlines() if changed_var in x]
    print("Before: %s" % ", ".join(status_old))
    print("After:  %s" % ", ".join(status_new))
    diff = list(context_diff(status_old, status_new))
    difference = "\n".join([x for x in diff if x.startswith("!")])

    if difference == "":
        error = 'WARN: Modifying VDO to %s did nothing.' % changed_argument
        print(error)
        errors.append(error)
    return


def minimum_slab_size(device, default_to_2g=True):
    def _get_raid_device(device):
        device_name = device.split("/").pop()
        ret, device_link = run(cmd="ls -al /dev/md | grep %s" % device_name, return_output=True)
        if ret or device_link is None:
            print("WARN: Device %s not found in /dev/md." % device_name)
            return None
        raid_device = device_link.split("../").pop()
        return raid_device

    if device.startswith("/dev/md"):
        device_name = _get_raid_device(device)
    else:
        device_name = device.split("/").pop()
    ret, device_size = run(cmd="lsblk | grep '%s ' " % device_name, return_output=True)
    if ret or device_size is None:
        print("WARN: Device %s not found using lsblk. Using default 2G size." % device_name)
        return "2G"
    size = device_size.split()[3]
    multipliers = ["M", "G", "T", "P", "E"]
    device_size = (float(size[:-1]) * (1024 ** multipliers.index(size[-1:]))).__int__()
    max_number_of_slabs = 8192
    minimum_size = 2 ** int((device_size / max_number_of_slabs)).bit_length()
    if minimum_size < 128:
        minimum_size = 128
    if default_to_2g and minimum_size < 2048:
        return "2G"
    return str(minimum_size) + "M"


def maximum_logical_size():
    """
    Returns maximum logical size based on memory
    :return: string max_size
    """
    memory = get_memory()['mem']['free']
    size = '4096T'
    if memory < 10000:
        size = 2 ** (((4096.0 / 10000) * float(memory)).__int__().bit_length() - 1)
        if size > 4096:
            size = 4096
        size = str(size) + 'T'
    return size


def is_block_device(device):
    try:
        mode = os.stat(device).st_mode
    except OSError:
        msg = "Device %s does not exist." % device
        return msg

    if not stat.S_ISBLK(mode):
        msg = "Device %s is not block device, aborting." % device
        print(msg)
        return msg
    return True


def get_underlying_device(name, conf_file="/etc/vdoconf.yml"):
    vdo = VDO(disable_check=True)
    ret, data = vdo.status(name=name, return_output=True, verbosity=False, conf_file=conf_file)
    if ret != 0:
        msg = "FAIL: Could not get status of VDO device '%s'." % name
        print(msg)
        return None
    device = None
    for line in data.splitlines():
        if 'Storage device' in line:
            device = "/dev/%s" % line.split("/dev/").pop().split().pop(0).strip()
    if device is None:
        # The device is probably crashed and needs to be force removed
        print("WARN: Could not find 'Device mapper status' in vdo status output.")
        # Let's try alternative way of getting the device by checking vdo config file
        # First get the config file
        conf_file = None
        for line in data.splitlines():
            if "File:" in line:
                conf_file = line.split("File:").pop().strip()
        if not conf_file:
            print("FAIL: Could not find vdo conf file in vdo status, something is really wrong!")
            return None
        # Read the file contents
        with open(conf_file) as f:
            lines = f.readlines()
        correct_vdo_device = False
        for line in lines:
            if "%s:" % name in line:
                # Get the correct VDO device in the config, there might be more
                correct_vdo_device = True
            if correct_vdo_device and "device:" in line:
                # Now we have the device, just need to follow the link
                device = line.split("device:").pop().strip()
                break

    # now we might have /dev/disk/by-id/UUID, need to get something reasonable
    _, data = run("ls -la %s" % device, return_output=True, verbose=False)
    device = data.split("/").pop().strip()

    # format dm-X causes issues later on, where the device cannot be found using lsblk to check for size
    if device.startswith("dm-"):
        with open("/sys/block/%s/dm/name" % device) as f:
            dev_name = f.readline().rstrip('\n')
        device = "/dev/mapper/%s" % dev_name
    else:
        device = "/dev/%s" % device

    return device


def get_replace_dict():
    """
    Returns dict of keys to replace from fmf to libsan.host.vdo.VDO
    :return: dict
    """
    return {"vdo_name": 'name'}


def get_vdo_args(vdo_object):
    """
    Returns dict of arguments passed from FMF env vars
    :return: dict
    """
    arguments = dict(vdo_object.arguments)
    # "name" has to be removed, as name is FMF name of the test at this point
    del (arguments["name"])
    return get_env_args(list(arguments) + list(get_replace_dict()) + ["grow_type"],
                        read_var("FILE_NAMES"),
                        get_replace_dict())



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def fs_resize_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    old_fs_size = read_var("LSM_FS_SIZE")
    new_fs_size = environ['fmf_new_fs_size']

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Resizing FS %s to size %s with protocol %s." % (fs_id, new_fs_size, config['protocol']),
                   command=lsm.fs_resize,
                   fs=fs_id,
                   size=new_fs_size,
                   force=True,
                   errors=errors)

        # FIXME: Check the size is changed.

        atomic_run("Resizing FS %s back to size %s with protocol %s." % (fs_id, old_fs_size, config['protocol']),
                   command=lsm.fs_resize,
                   fs=fs_id,
                   size=old_fs_size,
                   force=True,
                   errors=errors)

    return errors


def fs_resize_fail():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail resizing without any paramethers with protocol %s" % config['protocol'],
             'command': lsm.fs_resize},
            {'message': "Trying to fail resizing FS with wrong 'fs' 'wrong' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': "wrong"},
            {'message': "Trying to fail resizing FS with 'fs' name instead of ID with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': fs_name},
            {'message': "Trying to fail resizing FS without 'size' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': fs_id},
            {'message': "Trying to fail resizing FS without 'fs' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'size': 10},
            {'message': "Trying to fail resizing FS with 'size' '-1' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': fs_id, 'size': -1},
            {'message': "Trying to fail resizing FS with 'size' '0' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': fs_id, 'size': 0},
            {'message': "Trying to fail resizing FS with 'size' 'B' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': fs_id, 'size': "B"}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       force=True,
                       **argument)
    return errors


def fs_resize_fail_not_enough_space():
    errors = []

    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail resizing FS with 'size' '8000P' with protocol %s" % config['protocol'],
             'command': lsm.fs_resize, 'fs': fs_id, 'size': "8000P"}
        ]
        for argument in arguments:
            atomic_run(expected_ret=4,
                       errors=errors,
                       force=True,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = fs_resize_success()
    if int(environ['fmf_tier']) == 2:
        errs = fs_resize_fail()
        errs += fs_resize_fail_not_enough_space()
    exit(parse_ret(errs))



#!/usr/bin/python
# Copyright (C) 2017 Red Hat, Inc.
# iscsi_params.py is a part of python-stqe
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Martin Hoyer   <mhoyer@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals

import argparse
import random
import sys
import libsan.host.linux as linux
import libsan.host.fio as fio
import libsan.host.iscsi as iscsi
import libsan.host.lio as lio
import libsan.host.mp as mp
import libsan.host.scsi as scsi
from six.moves import range
from six.moves import zip

from stqe.host.LogChecker import check_all

t_iqn = 'iqn.2017-11.com.redhat:params-test'  # Target IQN.
parameters = ['HeaderDigest', 'MaxRecvDataSegmentLength', 'MaxXmitDataSegmentLength', 'FirstBurstLength',
              'MaxBurstLength', 'ImmediateData', 'InitialR2T']  # Parameters to be changed.

# Same parameters in /etc/iscsid.conf. Using escape characters as a workaround for bug in Augeas iscsid lens.
iscsid_params = ['node.conn\[0\].iscsi.HeaderDigest', 'node.conn\[0\].iscsi.MaxRecvDataSegmentLength',
                 'node.conn\[0\].iscsi.MaxXmitDataSegmentLength', 'node.session.iscsi.FirstBurstLength',
                 'node.session.iscsi.MaxBurstLength', 'node.session.iscsi.ImmediateData',
                 'node.session.iscsi.InitialR2T']


def generate_values():  # Generating random values from the pools for both target and initiator.
    tgt_values = []
    in_values = []

    # Pools for random choice.
    digest = ['None', 'None,CRC32C', 'CRC32C,None']  # Separate 'CRC32C' option to prevent 'CRC32C'+'None' situations
    byte = ["512", "1024", "2048", "4096", "8192", "16384", "32768", "65536", "131072", "262144", "524288", "1048576",
            "16777212"]
    yesno = ['Yes', 'No']
    crc = random.getrandbits(2)  # Generates 0-3
    if linux.dist_ver() > 7:
        if crc == 0:
            tgt_values.append('CRC32C')
            in_values.append('CRC32C')
        else:
            tgt_values.append(random.choice(digest))
            in_values.append(random.choice(digest))

    for _ in range(4):
        tgt_values.append(random.choice(byte))
        in_values.append(random.choice(byte))

    for _ in range(2):
        tgt_values.append(random.choice(yesno))
        in_values.append(random.choice(yesno))

    return tgt_values, in_values


def discovery_login():  # Discovering and logging into targets on localhost. There should be only one.
    iscsi.discovery_st('127.0.0.1', disc_db=True, ifaces='default')

    if not iscsi.node_login(portal='127.0.0.1', udev_wait_time=5):
        print('FAIL: Unable log into the target')
        return False

    return True


def do_io(device):  # fio - running randwrite until the 256M device is full, then verifying written data using CRC32C.
    if not fio.install_fio():
        print('FAIL: Unable to install fio')
        return False

    fio_params = {'bs': '4k', 'direct': 1, 'ioengine': 'libaio', 'iodepth': 16, 'verify': 'crc32c', 'verify_fatal': 1,
                  'do_verify': 1, 'rw': 'randwrite', 'runtime': 120}  # adding runtime cap of 2 minutes

    if not fio.fio_stress(device, verbose=True, **fio_params):
        print('FIO I/O failed')
        return False

    return True


def cleanup():
    if not (iscsi.clean_up() and lio.lio_clearconfig()):
        print("FAIL: Unable to clean-up iSCSI environment")
        return False

    return True


def main():
    print("INFO: Checking logs before iscsi_params test")
    if not check_all():
        print("WARN: Detected an error on LogChecker")

    parser = argparse.ArgumentParser()
    # noinspection PyTypeChecker
    parser.add_argument("--iterations", "-i", default=10, required=False, dest="iterations",
                        help="number of iterations", type=int)

    args = parser.parse_args()

    if not iscsi.install():
        return False

    if linux.dist_ver() < 8:
        parameters.remove("HeaderDigest")
        iscsid_params.remove("node.conn\[0\].iscsi.HeaderDigest")

    for _ in range(args.iterations):
        target_val, initiator_val = generate_values()
        tgt_params_values = dict(list(zip(parameters, target_val)))
        in_params_values = dict(list(zip(iscsid_params, initiator_val)))
        iscsi.clean_up()

        # Installing targetcli, clearing previous configuration, creating 256M local LIO target.
        if not lio.lio_setup_iscsi_target(lun_size='256M', tgt_iqn=t_iqn):
            return False

        # Setting parameter values with targetcli.
        for param in tgt_params_values:
            if not lio.lio_iscsi_target_set_parameter(tgt_iqn=t_iqn, tpg='tpg1', group='parameter', attr_name=param,
                                                      attr_value=tgt_params_values[param]):
                return False

        if not iscsi.set_iscsid_parameter(**in_params_values):  # Setting parameter values in /etc/iscsid.conf.
            return False

        if not discovery_login():
            return False

        # How to check params manually:
        # iscsiadm -m session -P2 | grep HeaderDigest | cut -d " " -f 2
        # cat /sys/class/iscsi_connection/connection*/header_digest
        # cat /sys/class/iscsi_session/session*/first_burst_len

        # Printing negotiated values.
        sid = iscsi.get_all_session_ids()
        if not sid:
            print("FAIL: Could not find session ID to use...")
            return False
        negotiated = iscsi.query_iscsi_session(sid[0])

        session_info_dict = {'HeaderDigest': 'header_digest', 'MaxRecvDataSegmentLength': 'max_recv',
                             'MaxXmitDataSegmentLength': 'max_xmit', 'FirstBurstLength': 'first_burst',
                             'MaxBurstLength': 'max_burst', 'ImmediateData': 'immediate_data',
                             'InitialR2T': 'initial_r2t'}

        count = 0
        for p in parameters:
            t_value = target_val[count]
            i_value = initiator_val[count]
            n_value = negotiated[session_info_dict[p]]
            n_expected = None
            print(p + ':')
            print('Target: %-11s| Initiator: %-11s| Negotiated: %-11s' % (t_value, i_value, n_value))
            print('______________________________________________________________\n')

            # Check if negotiated values are correct
            if p == 'HeaderDigest':
                if i_value.split(',')[0] == 'CRC32C':  # Initiator preference has priority.
                    n_expected = 'CRC32C'
                else:
                    n_expected = 'None'

                if (i_value == 'CRC32C,None') and (t_value == 'None'):
                    n_expected = 'None'

            elif p == 'MaxRecvDataSegmentLength':
                n_expected = i_value

            elif (p == 'MaxXmitDataSegmentLength') or (p == 'MaxBurstLength') or (p == 'FirstBurstLength'):
                if int(t_value) < int(i_value):
                    n_expected = t_value
                else:
                    n_expected = i_value

            elif p == 'ImmediateData':
                if (t_value == 'Yes') and (i_value == 'Yes'):
                    n_expected = 'Yes'
                else:
                    n_expected = 'No'

            elif p == 'InitialR2T':
                if (t_value == 'No') and (i_value == 'No'):
                    n_expected = 'No'
                else:
                    n_expected = 'Yes'

            if p == 'FirstBurstLength':
                # FirstBurstLength cannot be higher than MaxBurstLength
                if int(target_val[count + 1]) < int(initiator_val[count + 1]):
                    exp_max_burst = target_val[count + 1]
                else:
                    exp_max_burst = initiator_val[count + 1]

                if int(n_expected) > int(exp_max_burst):
                    n_expected = exp_max_burst

            if n_expected != n_value:
                print("FAIL: expected: %s, negotiated: %s" % (n_expected, n_value))
                return False

            count += 1

        # Choosing device to run IO on.
        test_dev = None
        if linux.is_service_running(mp.mp_service_name()):
            mpaths = mp.get_free_mpaths()
            if mpaths:
                test_dev = '/dev/mapper/' + list(mpaths.keys())[0]
            else:
                print("FAIL: multipathd is running but no free mpath devices found.")
                file = open('/etc/multipath.conf', 'r')
                print('/etc/multipath.conf:\n\n', file.read())
                return False
        else:
            dev = scsi.get_free_disks(filter_only={'model': 'params-test_lun'})
            if not dev:
                print("FAIL: Could not find device to use...")
                return False
            test_dev = '/dev/' + list(dev.keys())[0]

        if not do_io(test_dev):
            return False

        if not check_all():
            print("FAIL: Detected error on LogChecker")
            return False


if main() is False:
    cleanup()
    sys.exit(1)

if not cleanup():
    sys.exit(1)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.fio import fio_stress
from stqe.host.atomic_run import atomic_run, parse_ret
from os import environ
from libsan.host.stratis import Stratis
from stqe.host.persistent_vars import read_var
from libsan.host.linux import mkdir, rmdir, mount, umount


def basic_stress():
    errors = []

    stratis = Stratis()
    pool_name = "pool_stress"
    fs_name = "fs_stress"
    mnt_dir = "basic_stress"
    stratis_fs_path = f"/dev/stratis/{pool_name}/{fs_name}"
    blockdev = read_var("STRATIS_DEVICE")

    atomic_run("Creating pool",
               pool_name=pool_name,
               blockdevs=blockdev,
               command=stratis.pool_create,
               errors=errors
               )

    atomic_run("Creating FS",
               pool_name=pool_name,
               fs_name=fs_name,
               command=stratis.fs_create,
               errors=errors
               )

    atomic_run(f"Creating dir: {mnt_dir}",
               new_dir=mnt_dir,
               command=mkdir,
               errors=errors
               )

    atomic_run(f"Mounting filesystem: {fs_name}",
               device=stratis_fs_path,
               mountpoint=mnt_dir,
               command=mount,
               errors=errors
               )

    arguments = [
        {'message': "Checking stratis status to get info on data.", 'command': stratis.fs_list},
        # FIXME: Get max size of device to run stress test
        {'message': "Starting fio stress on stratis fs",
         'command': fio_stress,
         'directory': f"{mnt_dir}",
         'of': None,
         'verbose': True,
         # fio settings
         'ioengine': environ['fmf_ioengine'],
         'direct': environ['fmf_direct'],
         'group_reporting': environ['fmf_group_reporting'],
         'bs': environ['fmf_bs'],
         'iodepth': environ['fmf_iodepth'],
         'numjobs': environ['fmf_numjobs'],
         'rw': environ['fmf_rw'],
         'rwmixread': environ['fmf_rwmixread'],
         'randrepeat': environ['fmf_randrepeat'],
         'norandommap': environ['fmf_norandommap'],
         'size': environ['fmf_size'],
         'runtime': int(environ['fmf_runtime'])*60},
        {'message': "Checking stratis status to get info on data.", 'command': stratis.fs_list}
    ]

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)

    atomic_run("Unmounting filesystem",
               mountpoint=mnt_dir,
               command=umount,
               errors=errors
               )

    atomic_run("Removing mnt dir",
               dir_name=mnt_dir,
               command=rmdir,
               errors=errors
               )

    atomic_run(f"Destroying filesystem: fs{fs_name}",
               pool_name=pool_name,
               fs_name=fs_name,
               command=stratis.fs_destroy,
               errors=errors
               )

    atomic_run(f"Destroying pool: {pool_name}",
               pool_name=pool_name,
               command=stratis.pool_destroy,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = basic_stress()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2019 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""stratis.py: Module with test specific method for Stratis."""

from stqe.host.fmf_tools import get_env_args
from stqe.host.persistent_vars import read_var


def get_stratis_args(stratis_object):
    """
    Returns dict of arguments passed from FMF env vars
    :return: dict
    """
    arguments = dict(stratis_object.arguments)
    # Stratis does not use kwargs, only positional ones. SO we need to add them here manually.
    positional_args = ["pool_name", "blockdevs", "current", "new", "fs_name", "origin_name", "snapshot_name",
                       "new_name", "key_desc", "keyfile_path", "cmd", "clevis", "thumbprint", "tang_url", "trust_url",
                       "binding_method", "pool_error_code", "pool_uuid", "debug_subcommand", "fs_size", "fs_amount",
                       "pool_overprovision", "no_overprovision", "fs_amount", "unlock_method", "stopped_pools", ]
    return get_env_args(list(arguments) + list(positional_args),
                        read_var("FILE_NAMES"),
                        {})



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from libsan.host.scsi import get_scsi_disk_ids
from libsan.host.lio import TargetCLI
from libsan.host.linux import wait_udev
from os import environ


def create_tpg_lun():
    errors = []
    disk_id = None

    target = TargetCLI(path="/loopback")

    lun = environ["fmf_lun"]
    storage_object = read_var("FILEIO_NAME")

    ret, data = atomic_run("Creating loopback",
                           return_output=True,
                           command=target.create,
                           errors=errors
                           )

    wwn = data.split()[2][:-1]

    target.path = "/loopback/" + wwn + "/luns"

    atomic_run("Creating lun with name: %s" % lun,
               command=target.create,
               storage_object="/backstores/fileio/" + storage_object,
               lun=lun,
               errors=errors
               )

    atomic_run("Waiting udev to finish storage scan",
               command=wait_udev,
               errors=errors
               )

    scsi_ids = get_scsi_disk_ids()

    for scsi_id in scsi_ids:
        if lun == scsi_id.split(":")[3]:
            disk_id = scsi_id

    atomic_run("Writing var SCSI_ID",
               command=write_var,
               var={"SCSI_ID": disk_id},
               errors=errors)

    atomic_run("Writing var LOOPBACK_WWN",
               command=write_var,
               var={"LOOPBACK_WWN": wwn},
               errors=errors)

    atomic_run("Writing var LOOPBACK_LUN",
               command=write_var,
               var={"LOOPBACK_LUN": lun},
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = create_tpg_lun()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_unmask_fail():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail unmasking access group to volume without any paramethers with protocol %s"
                        % config['protocol'], 'command': lsm.volume_unmask},
            {'message': "Trying to fail unmasking access group to volume without volume with protocol %s"
                        % config['protocol'], 'ag': ag_id, 'command': lsm.volume_unmask},
            {'message': "Trying to fail unmasking access group to volume without access group with protocol %s"
                        % config['protocol'], 'vol': vol_id, 'command': lsm.volume_unmask},
            {'message': "Trying to fail unmasking access group to volume with wrong volume with protocol %s"
                        % config['protocol'], 'ag': ag_id, 'vol': "WRONG", 'command': lsm.volume_unmask},
            {'message': "Trying to fail unmasking access group to volume with wrong access group with protocol %s"
                        % config['protocol'], 'vol': vol_id, 'ag': "WRONG", 'command': lsm.volume_unmask}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def volume_unmask_no_state_change():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Removing access for access group %s to volume %s to hit NO_STATE_CHANGE with protocol %s." %
                        (ag_id, vol_id, config['protocol']), 'vol': vol_id, 'ag': ag_id, 'command': lsm.volume_unmask}
        ]
        for argument in arguments:
            atomic_run(expected_ret=4,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 2:
        errs = volume_unmask_fail()
        errs + volume_unmask_no_state_change()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thinp test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        loop_dev["dev%s" % dev_num] = loopdev.create_loopdev(size=512)
        if not loop_dev["dev%s" % dev_num]:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False

    pvs = "%s %s %s %s" % (loop_dev["dev1"], loop_dev["dev2"], loop_dev["dev3"], loop_dev["dev4"])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    # Test thin pool creation with different options
    TestObj.tok("lvcreate -l1 --thin %s/pool1" % vg_name)
    TestObj.tok("lvcreate -l1 -T %s" % vg_name)
    TestObj.tok("lvcreate -l1 -V2G -T %s" % vg_name)
    TestObj.tok("lvcreate -L4M -V2G --thin %s/pool2" % vg_name)
    TestObj.tok("lvcreate -L4M -V2G -T %s/pool3 -n lv1" % vg_name)
    TestObj.tok("lvcreate -L4M -V2G -T %s/pool4 -n %s/lv2" % (vg_name, vg_name))
    TestObj.tok("lvremove -ff %s" % vg_name)

    # Create thin LV in existing pool
    TestObj.tok("lvcreate -L4M -T %s/pool" % vg_name)
    # create thin lv with 2G, lvm will name the LV
    TestObj.tok("lvcreate -V2G -T %s/pool" % vg_name)
    TestObj.tok("lvcreate -V2G -T %s/pool -n lv1" % vg_name)
    TestObj.tok("lvcreate -V2G -T %s/pool -n %s/lv2" % (vg_name, vg_name))
    TestObj.tok("lvremove -ff %s" % vg_name)

    # Test --type thin|thin-pool
    TestObj.tok("lvcreate -l1 --type thin %s/pool1 --virtualsize 1G" % vg_name)
    TestObj.tok("lvcreate -l1 --type thin-pool %s/pool2" % vg_name)
    TestObj.tlog("Test RHEL6.6 --type thin bug 1176006")
    TestObj.tnok("lvcreate -l1 --type thin %s/pool" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    # Test --thinpool name/path
    TestObj.tok("lvcreate -l1 -T --thinpool %s/pool3" % vg_name)
    TestObj.tok("lvcreate -l1 --thinpool %s/pool4" % vg_name)
    TestObj.tok("lvcreate -l1 --thinpool pool5 %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    #
    # Test if LVM lv_metadata_size is correct
    #
    TestObj.tok("lvcreate -L8M -c 256 -T %s/pool1" % vg_name)
    lv_info = lvm.lv_info("pool1", vg_name, options="lv_metadata_size")
    if lv_info["lv_metadata_size"] == "4.00m":
        TestObj.tpass("%s/pool1 lv_metadata_size == 4.00m" % vg_name)
    else:
        TestObj.tfail("%s/pool1 lv_metadata_size == %s, but expected 4.00m" % (vg_name, lv_info["lv_metadata_size"]))

    TestObj.tok("lvcreate -L16M --poolmetadatasize 8 -T %s/pool2" % vg_name)
    lv_info = lvm.lv_info("pool2", vg_name, options="lv_metadata_size")
    if lv_info["lv_metadata_size"] == "8.00m":
        TestObj.tpass("%s/pool2 lv_metadata_size == 8.00m" % vg_name)
    else:
        TestObj.tfail("%s/pool2 lv_metadata_size == %s, but expected 8.00m" % (vg_name, lv_info["lv_metadata_size"]))

    TestObj.tok("lvremove -ff %s" % vg_name)

    #
    # Test --chunksize, -c
    #
    # small chunk size (min is 64KB)
    TestObj.tok("lvcreate --chunksize 64 -l1 -T %s/pool1" % vg_name)
    lv_info = lvm.lv_info("pool1", vg_name, options="chunksize")
    if lv_info["chunksize"] == "64.00k":
        TestObj.tpass("%s/pool1 chunksize == 64.00k" % vg_name)
    else:
        TestObj.tfail("%s/pool1 chunksize == %s, but expected 64.00k" % (vg_name, lv_info["chunksize"]))
    TestObj.tnok("lvcreate -c 32 -l1 -T %s/pool2" % vg_name)

    # big chunk size (max is 1048576 - 1G)
    TestObj.tok("lvcreate --chunksize 1048576 -L1g -T %s/pool2" % vg_name)
    lv_info = lvm.lv_info("pool2", vg_name, options="chunksize")
    if lv_info["chunksize"] == "1.00g":
        TestObj.tpass("%s/pool2 chunksize == 1.00g" % vg_name)
    else:
        TestObj.tfail("%s/pool2 chunksize == %s, but expected 1.00g" % (vg_name, lv_info["chunksize"]))
    TestObj.tnok("lvcreate -c 2097152 -L2g -T %s/pool3" % vg_name)

    # Check if chunk size is correct
    TestObj.tok("lvcreate -l1 -c 512 -T %s/pool3" % vg_name)
    lv_info = lvm.lv_info("pool3", vg_name, options="chunksize")
    if lv_info["chunksize"] == "512.00k":
        TestObj.tpass("%s/pool3 chunksize == 512.00k" % vg_name)
    else:
        TestObj.tfail("%s/pool3 chunksize == %s, but expected 512.00k" % (vg_name, lv_info["chunksize"]))
    TestObj.tok("lvremove -ff %s" % vg_name)

    #
    # Test --extents -l %FREE, VG, PVS
    #
    TestObj.tok("lvcreate --extents 10%%VG -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -l 10%%PVS -T %s/pool2" % vg_name)
    TestObj.tok("lvcreate -l 10%%PVS -T %s/pool4" % vg_name)
    TestObj.tok("lvcreate -l 10%%FREE -T %s/pool3" % vg_name)
    TestObj.tok("lvcreate -l 100%%FREE -T %s/pool5" % vg_name)
    if not lvm.vg_show():
        TestObj.tfail("Could not show VG %s" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    TestObj.tok("lvcreate -l 100%%VG -T %s/pool1" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)
    TestObj.tok("lvcreate -l 100%%PVS -T %s/pool1" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    # test with invalid option
    TestObj.tnok("lvcreate --extents 10%%test -T %s/pool1" % vg_name)

    # Since RHEL-6.6 LVM improved usage of percentage allocation
    # lvcreate man page: When expressed as a percentage, the number is  treated
    # as an approximate upper limit for the total number of physical extents to
    # be allocated (including extents used by any mirrors, for example).
    TestObj.tok("lvcreate --extents 110%%FREE -T %s/pool2" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    #
    # -virtualsize -V (using different size Unit)
    #
    TestObj.tok("lvcreate -l 90%%FREE -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V4096B -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V4096K -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V4096M -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V1G -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V1T -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -V15P -T %s/pool1" % vg_name)
    # Exceed maximum size
    TestObj.tnok("lvcreate -V16P -T %s/pool1" % vg_name)
    TestObj.tok("lvremove -ff %s" % vg_name)

    #
    # --discards
    #
    TestObj.tok("lvcreate -L92m -T %s/pool" % vg_name)
    # default discards is passdown
    lv_info = lvm.lv_info("pool", vg_name, options="discards")
    if lv_info["discards"] == "passdown":
        TestObj.tpass("%s/pool discards == passdown" % vg_name)
    else:
        TestObj.tfail("%s/pool1 discards == %s, but expected passdown" % (vg_name, lv_info["discards"]))

    TestObj.tok("lvcreate -L16m -T %s/pool1 --discards nopassdown" % vg_name)
    lv_info = lvm.lv_info("pool1", vg_name, options="discards")
    if lv_info["discards"] == "nopassdown":
        TestObj.tpass("%s/pool1 discards == nopassdown" % vg_name)
    else:
        TestObj.tfail("%s/pool1 discards == %s, but expected nopassdown" % (vg_name, lv_info["discards"]))

    TestObj.tok("lvcreate -L16m -T %s/pool2 --discards ignore" % vg_name)
    lv_info = lvm.lv_info("pool2", vg_name, options="discards")
    if lv_info["discards"] == "ignore":
        TestObj.tpass("%s/pool2 discards == ignore" % vg_name)
    else:
        TestObj.tfail("%s/pool2 discards == %s, but expected ignore" % (vg_name, lv_info["discards"]))

    TestObj.tok("lvcreate -L16m -T %s/pool3 --discards passdown" % vg_name)
    lv_info = lvm.lv_info("pool3", vg_name, options="discards")
    if lv_info["discards"] == "passdown":
        TestObj.tpass("%s/pool3 discards == passdown" % vg_name)
    else:
        TestObj.tfail("%s/pool3 discards == %s, but expected passdown" % (vg_name, lv_info["discards"]))

    TestObj.tok("lvremove -ff %s" % vg_name)

    #
    # test invalid option
    #
    # Thin pool mirror is not supported
    TestObj.tnok("lvcreate -l1 -m 1 -T %s/pool" % vg_name)

    TestObj.tok("lvcreate -L4M --thin %s/pool" % vg_name)
    TestObj.tnok("lvconvert -m 1 %s/pool" % vg_name)

    TestObj.tok("lvremove -ff %s" % vg_name)

    return True


def _clean_up():
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        print(e)
        TestObj.tfail("There was some problem while running the test (%s)" % e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, delete_loopdev
import libsan.host.lvm as lvm
import libsan.host.dm as dm
import libsan.host.linux as linux
import stqe.host.tc
import sys
import argparse

TestObj = None

loop_dev = {}

vg_name = "vgcache"
slow_lv = "slow_lv"
fast_lv = "cache_lv"
fast_meta_lv = "cache_metadata_lv"
mnt_point = "/mnt/cache_test"


def start_test(filesystem):
    global TestObj

    print(80 * "#")
    print("INFO: Starting test on FS %s" % filesystem)
    print(80 * "#")

    _clean_up()

    global loop_dev
    loop_dev["slow_dev"] = create_loopdev(size=4096)
    if not loop_dev["slow_dev"]:
        # Skip test if we are not able to create loopdev
        TestObj.tskip("Could not create loop device to be used as slow_dev")
        return 2

    loop_dev["fast_dev"] = create_loopdev(size=2048)
    if not loop_dev["fast_dev"]:
        TestObj.tskip("Could not create loop device to be used as fast_dev")
        return 2

    if not lvm.vg_create(vg_name, "%s %s" % (loop_dev["slow_dev"], loop_dev["fast_dev"])):
        TestObj.tfail("Could not create VG \"%s\"" % vg_name)
        return 1 

    print("INFO: Creating Slow LV")
    if run("lvcreate -n %s -L 3G %s %s" % (slow_lv, vg_name, loop_dev["slow_dev"])) != 0:
        TestObj.tfail("Could not create %s" % slow_lv)
        return 1 
    print("INFO: Creating Cache LV")
    if run("lvcreate -n %s -L 1G %s %s" % (fast_lv, vg_name, loop_dev["fast_dev"])) != 0:
        TestObj.tfail("Could not create %s" % fast_lv)
        return 1 
    print("INFO: Creating  Cache metadata LV")
    if run("lvcreate -n %s -L 12M %s %s" % (fast_meta_lv, vg_name, loop_dev["fast_dev"])) != 0:
        TestObj.tfail("Could not create %s" % fast_meta_lv)
        return 1 

    print("INFO: Creating cache pool logical volume")
    if run("lvconvert --yes --type cache-pool --cachemode writethrough --poolmetadata %s/%s %s/%s"
           % (vg_name, fast_meta_lv, vg_name, fast_lv)) != 0:
        TestObj.tfail("Could not create cache pool LV")
        return 1 

    print("INFO: Creating cache logical volume")
    if run("lvconvert --yes --type cache --cachepool %s/%s %s/%s"
           % (vg_name, fast_lv, vg_name, slow_lv)) != 0:
        TestObj.tfail("Could not create cache LV")
        return 1 

    print("INFO: Displaying LVs")
    run("lvs -a -o +devices")

    dm.dm_show_table()

    lv_device = "/dev/mapper/%s-%s" % (vg_name, slow_lv)
    print("INFO:Going to create FS(%s) on %s" % (filesystem, lv_device))
    if run("mkfs.%s %s" % (filesystem, lv_device)) != 0:
        TestObj.tfail("Could not create FS(%s) on %s" % (filesystem, lv_device))
        return 1 

    if not linux.mkdir(mnt_point):
        TestObj.tfail("Could not create directory %s - FS(%s)" % (mnt_point, filesystem))
        return 1 

    if not linux.mount(lv_device, mnt_point):
        TestObj.tfail("Could not mount %s - FS(%s)" % (mnt_point, filesystem))
        return 1 

    print("INFO: Going create file on %s" % mnt_point)
    filename = "file1.img"
    if run("dd conv=fdatasync if=/dev/urandom of=%s/%s bs=1M count=2000" % (mnt_point, filename)) != 0:
        TestObj.tfail("Could not create file on %s - FS(%s)" % (mnt_point, filesystem))
        return 1 

    linux.sync()
    run("lvs -a")

    print(80 * "#")
    TestObj.tpass("PASS: Test on FS %s" % filesystem)
    print(80 * "#")

    return 0


def _clean_up():
    global vg_name, loop_dev

    linux.umount(mnt_point)

    # make sure any failed device is removed
    run("dmsetup remove_all")
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if loop_dev[dev]:
                if not lvm.pv_remove(loop_dev[dev]):
                    TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
                linux.sleep(1)
                if not delete_loopdev(loop_dev[dev]):
                    TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    parser = argparse.ArgumentParser(description='cache_basic')
    parser.add_argument("--filesystem", "-f", required=False, dest="fs",
                        help="Filesystem name", metavar="filesytem")

    args = parser.parse_args()

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    filesystem = linux.get_default_fs()

    if args.fs:
        filesystem = args.fs

    ret = start_test(filesystem)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    if ret == 2:
        print("SKIP: Test has been skipped because loop device could not be created!")
        sys.exit(2)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import os
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from libsan.host.lio import TargetCLI


def create_pscsi():
    errors = []

    target = TargetCLI(path="/backstores/pscsi")

    scsi_id = read_var("SCSI_ID")
    name = os.environ["fmf_pscsi_name"]

    atomic_run("Creating backstore pscsi object with name: %s" % name,
               name=name,
               dev=scsi_id,
               command=target.create,
               errors=errors
               )

    atomic_run("Deleting pscsi object %s" % name,
               name=name,
               command=target.delete,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = create_pscsi()
    exit(parse_ret(errs))



#!/usr/bin/python

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import stqe.host.tc
from stqe.host.atomic_run import atomic_run
from libsan.host.cmdline import run
from libsan.host.dmpd import thin_restore, thin_check, thin_ls, thin_dump, thin_rmap, thin_delta, thin_repair, \
    thin_trim, get_help, get_version
from libsan.host.linux import install_package
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import vg_create, vg_remove, lv_create, lv_remove, lv_activate, lv_deactivate, lv_convert

TC = None


def _mount_thin_lv(vg_name, lv_name):
    ret = run("mkdir -p /mnt/%s" % lv_name, verbose=True)
    ret += run("mkfs.ext4 /dev/%s/%s" % (vg_name, lv_name), verbose=True)
    ret += run("mount /dev/%s/%s /mnt/%s/" % (vg_name, lv_name, lv_name), verbose=True)
    return ret


def _unmount_thin_lv(lv_name):
    ret = run("umount -f /mnt/%s/" % lv_name, verbose=True)
    ret += run("rm -rf /mnt/%s" % lv_name, verbose=True)
    return ret


def _metadata_snapshot(vg_name, lv_name):
    ret = run("dmsetup suspend /dev/mapper/%s-%s-tpool" % (vg_name, lv_name), verbose=True)
    ret += run("dmsetup message /dev/mapper/%s-%s-tpool 0 reserve_metadata_snap" % (vg_name, lv_name), verbose=True)
    ret += run("dmsetup resume /dev/mapper/%s-%s-tpool" % (vg_name, lv_name), verbose=True)
    return ret


def init(args):
    print("INFO: Initializing test case")
    errors = []

    atomic_run("Creating loopdev",
               expected_ret="/dev/loop1",
               name=args["loop"],
               size=args["loop_size"],
               command=create_loopdev,
               errors=errors)

    atomic_run("Creating VG",
               vg_name=args["group"],
               pv_name="/dev/" + args["loop"],
               command=vg_create,
               errors=errors)

    atomic_run("Creating thin pool",
               vg_name=args["group"],
               lv_name=args["pool"],
               options=["-T", "-L 100"],
               command=lv_create,
               errors=errors)

    # create few LVs to increase transaction ID and be able to do thin_delta
    for i in range(args["number of vols"]):
        atomic_run("Creating thin LV No. %s" % i,
                   vg_name=args["group"] + "/" + args["pool"],
                   lv_name=args["vol"] + str(i),
                   options=["-T", "-V 10"],
                   command=lv_create,
                   errors=errors)

        atomic_run("Mounting LV No. %s" % i,
                   expected_ret=0,
                   vg_name=args["group"],
                   lv_name=args["vol"] + str(i),
                   command=_mount_thin_lv,
                   errors=errors)

        atomic_run("Unmounting LV No. %s" % i,
                   expected_ret=0,
                   lv_name=args["vol"] + str(i),
                   command=_unmount_thin_lv,
                   errors=errors)

        atomic_run("Deactivating thin LV No. %s" % i,
                   lv_name=args["vol"] + str(i),
                   vg_name=args["group"],
                   command=lv_deactivate,
                   errors=errors)

    atomic_run("Creating metadata snapshot",
               expected_ret=0,
               lv_name=args["pool"],
               vg_name=args["group"],
               command=_metadata_snapshot,
               errors=errors)

    atomic_run("Deactivating pool",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Creating swap LV",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-L 100"],
               command=lv_create,
               errors=errors)

    atomic_run("Deactivating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Swapping metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--thinpool " + args["group"] + "/" + args["pool"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Activating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Initialization passed")
    else:
        TC.tfail("Initialization failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def clean(args):
    print("INFO: Cleaning up")
    errors = []

    # restoring metadata device in case it is corrupted
    atomic_run("Repairing metadata device",
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               quiet=True,
               command=thin_restore,
               errors=errors)

    # thinpool got activated after checking its metadata to get bad checksum
    atomic_run("Deactivating pool",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Deactivating swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Swapping back metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--thinpool " + args["group"] + "/" + args["pool"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Removing swap",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_remove,
               errors=errors)

    atomic_run("Removing thinpool",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_remove,
               errors=errors)

    atomic_run("Removing VG",
               vg_name=args["group"],
               force=True,
               command=vg_remove,
               errors=errors)

    atomic_run("Deleting loopdev",
               name=args["loop"],
               command=delete_loopdev,
               errors=errors)

    atomic_run("Deleting metadata file",
               cmd="rm -f /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Deleting repair metadata file",
               cmd="rm -f /var/tmp/metadata_repair",
               command=run,
               errors=errors)

    atomic_run("Deleting snapshot metadata file",
               cmd="rm -f /var/tmp/metadata_snap",
               command=run,
               errors=errors)

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def install_packages(packages):
    for package in packages:
        install_package(package)


def runtime_test(args):
    print("\n#######################################\n")
    print("INFO: Testing thin tools runtime provided by device_mapper_persistent_data, iteration No. %s" %
          args["iteration"])

    errors = []

    atomic_run("Checking metadata",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_check,
               errors=errors)

    atomic_run("Checking metadata with few paramethers",
               source_vg=args["group"],
               source_lv=args["swap"],
               super_block_only=True,
               skip_mappings=True,
               ignore_non_fatal_errors=True,
               command=thin_check,
               errors=errors)

    atomic_run("Listing information about thin LVs",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_ls,
               errors=errors)

    atomic_run("Listing information about thin LVs without headers",
               source_vg=args["group"],
               source_lv=args["swap"],
               no_headers=True,
               command=thin_ls,
               errors=errors)

    atomic_run("Dumping metadata to standard output without mappings",
               formatting="human_readable",
               source_vg=args["group"],
               source_lv=args["swap"],
               skip_mappings=True,
               command=thin_dump,
               errors=errors)

    atomic_run("Dumping metadata to standard output",
               formatting="human_readable",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_dump,
               errors=errors)

    atomic_run("Dumping metadata to standard output from snapshot",
               formatting="human_readable",
               source_vg=args["group"],
               source_lv=args["swap"],
               snapshot=True,
               command=thin_dump,
               errors=errors)

    atomic_run("Dumping metadata with dev-id",
               formatting="human_readable",
               source_vg=args["group"],
               source_lv=args["swap"],
               dev_id=args["number of vols"] - 1,
               command=thin_dump,
               errors=errors)

    atomic_run("Calculating metadata size for pool of 64k blocks and 100M size",
               cmd="thin_metadata_size -b64k -s100m -m1 -um",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for pool of 64k blocks and 100M size",
               cmd="thin_metadata_size -b64k -s100m -m1 -um -n",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for pool of 64k blocks and 100M size",
               cmd="thin_metadata_size -b64k -s100m -m1 -um -nlong",
               command=run,
               errors=errors)

    atomic_run("Calculating metadata size for pool of 64k blocks and 100M size",
               cmd="thin_metadata_size -b64k -s100m -m1 -um -nshort",
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device",
               source_vg=args["group"],
               source_lv=args["swap"],
               region="0..-1",
               command=thin_rmap,
               errors=errors)

    atomic_run("Dumping metadata to file",
               formatting="xml",
               source_vg=args["group"],
               source_lv=args["swap"],
               repair=True,
               output="/var/tmp/metadata",
               command=thin_dump,
               errors=errors)

    atomic_run("Dumping metadata to file from snapshot",
               formatting="xml",
               source_vg=args["group"],
               source_lv=args["swap"],
               snapshot=True,
               output="/var/tmp/metadata_snap",
               command=thin_dump,
               errors=errors)

    atomic_run("Getting differences between thin LVs",
               source_vg=args["group"],
               source_lv=args["swap"],
               thin1=1,
               thin2=args["number of vols"] - 1,
               snapshot=True,
               command=thin_delta,
               errors=errors)

    atomic_run("Getting differences between thin LVs with --verbose",
               source_vg=args["group"],
               source_lv=args["swap"],
               thin1=1,
               thin2=args["number of vols"] - 1,
               verbosity=True,
               snapshot=True,
               command=thin_delta,
               errors=errors)

    atomic_run("Getting differences between the same LV",
               source_vg=args["group"],
               source_lv=args["swap"],
               thin1=1,
               thin2=1,
               snapshot=True,
               command=thin_delta,
               errors=errors)

    atomic_run("Getting differences between the same LV with --verbose",
               source_vg=args["group"],
               source_lv=args["swap"],
               thin1=1,
               thin2=1,
               verbosity=True,
               snapshot=True,
               command=thin_delta,
               errors=errors)

    atomic_run("Listing metadata output from snapshot",
               source_vg=args["group"],
               source_lv=args["swap"],
               snapshot=True,
               command=thin_ls,
               errors=errors)

    # need to run everything on snapshot before this as thin_restore removes the metadata snapshot
    # Restoring metadata from snapshot is not supported, see BZ1477232
    atomic_run("Restoring metadata to get 'couldn't zero superblock'",
               False,
               source_file="/var/tmp/metadata_snap",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=thin_restore,
               errors=errors)

    atomic_run("Restoring metadata",
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=thin_restore,
               errors=errors)

    atomic_run("Repairing metadata from file to get 'Metadata is not large enough for superblock'",
               False,
               pkg=["device-mapper-persistent-data", "0.7.3-3"],
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=thin_repair,
               errors=errors)

    atomic_run("Repairing metadata to file",
               target_file="/var/tmp/metadata_repair",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_repair,
               errors=errors)

    atomic_run("Repairing metadata from file",
               source_file="/var/tmp/metadata_repair",
               target_vg=args["group"],
               target_lv=args["swap"],
               command=thin_repair,
               errors=errors)

    #   ##################### start of thin_trim #####################################
    # Need to swap back metadata to be able to activate thin pool
    atomic_run("Swapping metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--thinpool " + args["group"] + "/" + args["pool"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Activating swap for thin_trim",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)

    atomic_run("Activating pool for thin_trim",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)

    # thin_trim requires activated thin pool and its metadata, that was required beforehand
    atomic_run("Discarding free space of pool",
               pkg=["device-mapper-persistent-data", "0.7.3-3"],
               data_vg=args["group"],
               data_lv=args["pool"],
               metadata_file="/var/tmp/metadata_repair",
               command=thin_trim,
               errors=errors)

    atomic_run("Deactivating pool after thin_trim",
               lv_name=args["pool"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Deactivating swap after thin_trim",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_deactivate,
               errors=errors)

    atomic_run("Swapping back metadata",
               vg_name=args["group"],
               lv_name=args["swap"],
               options=["-y", "--thinpool " + args["group"] + "/" + args["pool"],
                        "--poolmetadata "],
               command=lv_convert,
               errors=errors)

    atomic_run("Activating swap after thin_trim",
               lv_name=args["swap"],
               vg_name=args["group"],
               command=lv_activate,
               errors=errors)
    #   ##################### end of thin_trim #####################################

    atomic_run("Checking metadata",
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_check,
               errors=errors)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing thin tools of device_mapper_persistent_data passed, iteration No. %s" % args["iteration"])
    else:
        TC.tfail("Testing thin tools of device_mapper_persistent_data failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]) +
                 "\nIteration No. %s" % args["iteration"])
        return 1
    return 0


def errors_test(args):
    print("\n#######################################\n")
    print("INFO: Testing thin tools errors provided by device_mapper_persistent_data, iteration No. %s" %
          args["iteration"])

    errors = []

    functions = ["thin_check", "thin_delta", "thin_dump", "thin_ls", "thin_metadata_size", "thin_repair",
                 "thin_restore", "thin_rmap", "thin_trim"]

    # Sanity to check for missing input
    for func in functions:
        atomic_run("Validating missing input",
                   False,
                   cmd=func,
                   command=run,
                   errors=errors)

    # Sanity to check with wrong input
    for func in functions:
        atomic_run("Validating wrong input",
                   False,
                   cmd=func + " wrong",
                   command=run,
                   errors=errors)

    # Sanity to check with wrong option
    for func in functions:
        atomic_run("Validating wrong option",
                   False,
                   cmd=func + " -wrong",
                   command=run,
                   errors=errors)

    # Sanity to check present functions with -h
    for func in functions:
        if func in ["thin_trim"]:
            kwargs = {'pkg': ["device-mapper-persistent-data", "0.7.3-2"]}
        else:
            kwargs = {}
        atomic_run("Checking help of command",
                   cmd=func,
                   command=get_help,
                   errors=errors,
                   **kwargs)

    # Sanity to check present functions with -V
    for func in functions:
        if func in ["thin_trim", "thin_metadata_size"]:
            kwargs = {'pkg': ["device-mapper-persistent-data", "0.7.3-2"]}
        else:
            kwargs = {}
        atomic_run("Checking version of command",
                   cmd=func,
                   command=get_version,
                   errors=errors,
                   **kwargs)

    atomic_run("Checking original pool metadata, should fail",
               False,
               source_vg=args["group"],
               source_lv=args["pool"],
               command=thin_check,
               errors=errors)

    atomic_run("Listing information about thin LVs",
               False,
               cmd="thin_ls /dev/mapper/%s-%s --format \"WRONG\"" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -b 64",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -b 64 -s 128",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -b 25 -s 128 -m 10",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -b 128 -s 64 -m 10",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -u h",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -n -n",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -nlongshort",
               command=run,
               errors=errors)

    atomic_run("Checking thin_metadata_size inputs",
               False,
               cmd="thin_metadata_size -b 128 -b 64",
               command=run,
               errors=errors)

    atomic_run("Repairing metadata without output",
               False,
               cmd="thin_repair -i /var/tmp/metadata_repair",
               command=run,
               errors=errors)

    atomic_run("Dumping metadata with wrong custom format",
               False,
               cmd="thin_dump /dev/mapper/%s-%s --format custom=wrong" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Dumping metadata with unknown format",
               False,
               cmd="thin_dump /dev/mapper/%s-%s --format wrong" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Dumping metadata with wrong dev-id",
               False,
               cmd="thin_dump /dev/mapper/%s-%s --dev-id wrong" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Repairing metadata to produce 'output file does not exist' error",
               False,
               cmd="thin_repair -i /dev/mapper/%s-%s -o /var/tmp/wrong.wrong" %
                   (args['group'], args['swap']),
               command=run,
               errors=errors)

    atomic_run("Repairing metadata to produce 'output file too small' error",
               False,
               cmd="thin_repair -i /var/tmp/metadata -o /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device, should fail without region",
               False,
               pkg=["device-mapper-persistent-data", "0.7.3-2"],
               cmd="thin_rmap /dev/mapper/%s-%s" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device with bad region to get 'badly formed region (end <= begin)",
               False,
               cmd="thin_rmap /dev/mapper/%s-%s --region 0..0" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device with bad region to get "
               "'badly formed region (couldn't parse numbers)'",
               False,
               cmd="thin_rmap /dev/mapper/%s-%s --region 0...1" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device with bad region to get 'badly formed region (no dots)'",
               False,
               cmd="thin_rmap /dev/mapper/%s-%s --region 00" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device with wrong path to get 'Couldn't stat path'",
               False,
               cmd="thin_rmap --region 0..-1 /var/tmp/wrong.wrong",
               command=run,
               errors=errors)

    atomic_run("Outputting reverse map of metadata device to get 'Metadata is not large enough for superblock'",
               False,
               cmd="thin_rmap --region 0..-1 /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Getting differences with thin1 ID out of range",
               False,
               source_vg=args["group"],
               source_lv=args["swap"],
               thin1=-1,
               thin2=args["number of vols"] - 1,
               command=thin_delta,
               errors=errors)

    atomic_run("Getting differences with thin2 ID out of range",
               False,
               source_vg=args["group"],
               source_lv=args["swap"],
               thin1=1,
               thin2=args["number of vols"] + 1,
               command=thin_delta,
               errors=errors)

    atomic_run("Restoring metadata without output",
               False,
               cmd="thin_restore -i /var/tmp/metadata",
               command=run,
               errors=errors)

    atomic_run("Restoring metadata with wrong options",
               False,
               cmd="thin_restore -i /var/tmp/metadata -o /dev/mapper/%s-%s --wrong test" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Restoring metadata with wrong source",
               False,
               cmd="thin_restore -i /var/tmp/wrong.wrong -o /dev/mapper/%s-%s" % (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Getting differences without thin2",
               False,
               cmd="thin_delta --thin1 1 /dev/mapper/%s-%s" %
                   (args["group"], args["swap"]),
               command=run,
               errors=errors)

    atomic_run("Corrupting metadata on device",
               cmd="echo 'nothing' >> /dev/mapper/%s-%s" %
                   (args['group'], args['swap']),
               command=run,
               errors=errors)

    atomic_run("Trying to fail while repairing metadata",
               False,
               source_vg=args['group'],
               source_lv=args['swap'],
               target_file="/var/tmp/metadata_repair",
               command=thin_repair,
               errors=errors)

    atomic_run("Trying to fail listing volumes",
               False,
               source_vg=args['group'],
               source_lv=args['swap'],
               command=thin_ls,
               errors=errors)

    atomic_run("Trying to fail while checking metadata",
               False,
               source_vg=args["group"],
               source_lv=args["swap"],
               command=thin_check,
               errors=errors)

    atomic_run("Trying to fail while dumping metadata from snapshot",
               False,
               formatting="human_readable",
               source_vg=args["group"],
               source_lv=args["swap"],
               snapshot=True,
               command=thin_dump,
               errors=errors)

    # restoring metadata device after corrupting it
    atomic_run("Repairing metadata device",
               source_file="/var/tmp/metadata",
               target_vg=args["group"],
               target_lv=args["swap"],
               quiet=True,
               command=thin_restore,
               errors=errors)

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing thin tools errors of device_mapper_persistent_data passed, iteration No. %s" %
                 args["iteration"])
    else:
        TC.tfail("Testing thin tools errors of device_mapper_persistent_data failed with following errors: \n\t'" +
                 "\n\t ".join([str(i) for i in errors]) +
                 "\nIteration No. %s" % args["iteration"])
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    args = {"loop": "loop1",
            "loop_size": 2048,
            "group": "vgtest",
            "pool": "thinpool",
            "vol": "thinvol",
            "number of vols": 10,
            "swap": "swapvol"}

    # Initialization
    packages = ["device-mapper-persistent-data"]
    install_packages(packages)

    init(args)

    # run it 3 times
    for i in range(1):
        args["iteration"] = i
        runtime_test(args)
        errors_test(args)

    clean(args)

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.vdo import VDO
from libsan.host.fio import fio_stress
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from os import environ
from stqe.host.vdo import is_block_device
from stqe.host.vdo import minimum_slab_size


def vdo_on_vdo_stress():
    errors = []
    vdo = VDO(disable_check=True)

    device = read_var("VDO_DEVICE")
    vdo_name = environ['fmf_vdo_name']

    ret = is_block_device("/dev/mapper/%s" % vdo_name)
    if ret != True:
        errors.append(ret)
        print(ret)
        return errors
    vdo_name = vdo_name + "_on_vdo"

    arguments = [
        {'message': "Creating VDO device on %s" % device, 'command': vdo.create, 'name': vdo_name, 'device': device},
        {'message': "Checking vdo status to get info on data.", 'command': vdo.status, 'name': vdo_name},
        {'message': "Starting fio stress on VDO on VDO",
         'command': fio_stress,
         'of': "/dev/mapper/%s" % vdo_name,
         'verbose': True,
         # fio settings
         'ioengine': environ['fmf_ioengine'],
         'direct': environ['fmf_direct'],
         'group_reporting': environ['fmf_group_reporting'],
         'bs': environ['fmf_bs'],
         'iodepth': environ['fmf_iodepth'],
         'numjobs': environ['fmf_numjobs'],
         'rw': environ['fmf_rw'],
         'rwmixread': environ['fmf_rwmixread'],
         'randrepeat': environ['fmf_randrepeat'],
         'norandommap': environ['fmf_norandommap'],
         'size': environ['fmf_size'],
         'runtime': int(environ['fmf_runtime'])*60},
        {'message': "Checking vdo status to get info on data.", 'command': vdo.status, 'name': vdo_name},
        {'message': "Removing VDO on VDO", "command": vdo.remove, 'name': vdo_name}
    ]


    min_slab_size = minimum_slab_size(device)
    if min_slab_size is not None:
        arguments[0]['slab_size'] = min_slab_size

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 2:
        errs = vdo_on_vdo_stress()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

import os
import libsan.host.linux as linux
import libsan.host.targetd as targetd
from time import sleep
from stqe.host.atomic_run import atomic_run, parse_ret


def targetd_ssl_test():
    errors = []

    arguments = dict()

    arguments["password"] = os.environ["fmf_ssl_password"]
    arguments["key"] = os.environ["fmf_key"]
    arguments["cert"] = os.environ["fmf_cert"]

    atomic_run("Starting targetd SSL test",
               enable=True,
               command=targetd.ssl_change,
               errors=errors
               )

    atomic_run("Adding password to the config",
               password=arguments["password"],
               command=targetd.set_password,
               errors=errors
               )

    atomic_run("Creating SSL key",
               name=arguments["key"],
               command=targetd.gen_ssl_key,
               errors=errors
               )

    atomic_run("Creating SSL certificate",
               name=arguments["cert"],
               key_name=arguments["key"],
               command=targetd.gen_ssl_cert,
               errors=errors
               )

    atomic_run("Targetd configured properly.... starting targetd service",
               service_name="targetd",
               command=linux.service_restart,
               errors=errors
               )

    print("Waiting 10 seconds for the service to restart")
    sleep(10)

    atomic_run("Getting service status",
               command=targetd.targetd_status,
               errors=errors
               )

    atomic_run("Checking TLS status",
               command=targetd.tls_status,
               errors=errors
               )

    atomic_run("Stopping targetd service",
               service_name="targetd",
               command=linux.service_stop,
               errors=errors
               )

    atomic_run("Removing password from config",
               command=targetd.set_password,
               errors=errors
               )

    atomic_run("Removing SSL cert: %s" % arguments["cert"],
               name=arguments["cert"] + ".pem",
               command=targetd.remove_file,
               errors=errors
               )

    atomic_run("Removing SSL key: %s" % arguments["key"],
               name=arguments["key"] + ".pem",
               command=targetd.remove_file,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = targetd_ssl_test()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.persistent_vars import read_var, clean_var
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI


def disable_chap():
    errors = []
    target_iqn = read_var("TARGET_IQN")
    tpg = read_var("TPG")

    target = TargetCLI(path="/iscsi")

    atomic_run("Disabling discovery_auth",
               enable=0,
               group="discovery_auth",
               command=target.set,
               errors=errors,
               )

    target.path = "/iscsi/" + target_iqn + "/" + tpg

    atomic_run("Disabling authentication",
               group="attribute",
               authentication=0,
               command=target.set,
               errors=errors
               )

    for i in ["CHAP_MUTUAL_USERID", "CHAP_MUTUAL_PASSWORD", "CHAP_PASSWORD", "CHAP_USERID"]:
        try:
            atomic_run("Removing var: %s from /tmp" % i,
                       var=i,
                       command=clean_var,
                       errors=errors
                       )
        except OSError:
            pass

    return errors


if __name__ == "__main__":
    errs = disable_chap()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def access_group_add_success():
    errors = []

    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID_2")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Adding initiator %s to AG %s with protocol %s." % (init_id, ag_id, config['protocol']),
                   command=lsm.access_group_add,
                   ag=ag_id,
                   init=init_id,
                   errors=errors)

        atomic_run("Removing initiator %s from AG %s with protocol %s." % (init_id, ag_id, config['protocol']),
                   command=lsm.access_group_remove,
                   ag=ag_id,
                   init=init_id,
                   force=True,
                   errors=errors)
    return errors


def access_group_add_fail():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail adding initiator to AG without any paramethers with protocol %s" % config[
                'protocol'],
             'command': lsm.access_group_add},
            {'message': "Trying to fail adding initiator to AG without AG with protocol %s" % config['protocol'],
             'command': lsm.access_group_add, 'init': init_id},
            {'message': "Trying to fail adding initiator to AG without init with protocol %s" % config['protocol'],
             'command': lsm.access_group_add, 'ag': ag_id},
            {'message': "Trying to fail adding initiator to AG with wrong init '-1' with protocol %s" % config[
                'protocol'], 'command': lsm.access_group_add, 'init': -1, 'ag': ag_id},
            {'message': "Trying to fail adding initiator to AG with NONEXISTENT AG with protocol %s" % config[
                'protocol'], 'command': lsm.access_group_add, 'init': init_id, 'ag': "NONEXISTENT"},
            {'message': "Trying to fail adding initiator to AG with NONEXISTENT init with protocol %s" % config[
                'protocol'], 'command': lsm.access_group_add, 'init': "NONEXISTENT", 'ag': ag_id},
            {'message': "Trying to fail adding initiator to AG with AG name instead of AG ID with protocol %s" % config[
                'protocol'], 'command': lsm.access_group_add, 'init': init_id, 'ag': ag_name}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def access_group_add_fail_no_state_change():
    errors = []

    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run("Adding initiator %s to AG %s to hit NO STATE CHANGE "
                   "with protocol %s." % (init_id, ag_id, config['protocol']),
                   command=lsm.access_group_add,
                   ag=ag_id,
                   init=init_id,
                   expected_ret=4,
                   errors=errors)
    return errors


def access_group_add_fail_exists_initiator():
    errors = []

    ag_name_2 = read_var("LSM_AG_NAME_2")
    sys_id = read_var("LSM_SYS_ID")
    init_id_2 = read_var("LSM_INIT_ID_2")
    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Creating AG_2 %s with protocol %s." % (ag_name_2, config['protocol']),
                               command=lsm.access_group_create,
                               name=ag_name_2,
                               init=init_id_2,
                               sys=sys_id,
                               return_output=True,
                               errors=errors)
        ag_id_2 = [line.split()[0].strip() for line in data.splitlines() if ag_name_2 in line][0]

        atomic_run("Adding initiator %s to AG %s to hit EXISTS_INITIATOR "
                   "with protocol %s." % (init_id, ag_id, config['protocol']),
                   command=lsm.access_group_add,
                   ag=ag_id,
                   init=init_id_2,
                   expected_ret=4,
                   errors=errors)

        atomic_run("Deleting AG_2 %s with protocol %s." % (ag_name_2, config['protocol']),
                   command=lsm.access_group_delete,
                   ag=ag_id_2,
                   force=True,
                   errors=errors)

    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = access_group_add_success()
    if int(environ['fmf_tier']) == 2:
        errs = access_group_add_fail()
        errs += access_group_add_fail_no_state_change()
        errs += access_group_add_fail_exists_initiator()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var
from libsan.host.linux import os_arch
from os import environ


def _is_wwpn(init):
    if len(init.split(":")) == 8:
        return True
    return False


def configure_lsm(protocols, username=None, password=None, target=None, **kwargs):
    print("INFO: Setting up libstoragemgmt for protocol %s." % protocols[0])
    arguments = locals()
    if 'kwargs' in arguments:
        kwargs = arguments.pop('kwargs')
        for kwarg in kwargs:
            arguments[kwarg] = kwargs[kwarg]

    errors = []

    for argument in arguments.keys():
        if arguments[argument] is None:
            continue

        var = "LSM_" + argument.upper()
        value = arguments[argument]
        # to write [val1, val2] instead of ['val1', 'val2']
        if isinstance(value, list):
            value = "[%s]" % ", ".join(value[:])
        atomic_run("Writing var %s" % var,
                   command=write_var,
                   var={'%s' % var: value},
                   errors=errors)

    return errors


if __name__ == "__main__":
    # list of variables to save in /tmp as persistent vars
    arguments = {
        'protocols': [],
        'username': None,
        'password': None,
        'target': None,
        'timeout': None,
        'pool_id': None,
        'pool_id_volume': None,
        'fs_size': None,
        'fs_export_path': None,
        'rw_host': None,
        'sys_id': None,
        'vol_size': None,
        'vol_size_2': None,
        'system_read_pct': None,
        'chap_in_pass': None,
        'chap_out_pass': None
    }
    for argument in arguments.keys():
        try:
            arguments[argument] = environ['fmf_' + argument]
        except KeyError:
            pass

    # these need to be arch specific
    arch = os_arch().replace("_", "-") # '_' is invalid in IQN
    arguments_arch = {
        'fs_name': None,
        'pool_name': None,
        'fs_cloned_name': None,
        'fs_snap_name': None,
        'ag_name': None,
        'ag_name_2': None,
        'init_id': None,
        'init_id_chap': None,
        'init_id_2': None,
        'vol_name': None,
        'vol_name_2': None,
        'vol_rep_name': None,
        'chap_in_user': None,
        'chap_out_user': None
    }
    for argument in arguments_arch.keys():
        try:
            arguments_arch[argument] = environ['fmf_' + argument]
            if 'init' in argument and _is_wwpn(arguments_arch[argument]):
                continue
            arguments_arch[argument] += "-%s" % arch
            # FS name cannot contain '-', INVALID_ARGUMENT(101)
            if any(x in argument for x in ['fs_name', 'fs_cloned_name', 'fs_snap_name']):
                arguments_arch[argument] = arguments_arch[argument].replace("-", "_")
        except KeyError:
            pass
    arguments.update(arguments_arch)

    try:
        if not arguments['protocols'].startswith("["):
            raise AttributeError
        arguments['protocols'] = arguments['protocols'][1:-1].split(", ")
    except AttributeError:
        print("ERROR: Protocols must be type list.")
        exit(1)

    errs = configure_lsm(**arguments)
    exit(parse_ret(errs))


#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.stratis import Stratis
from stqe.host.persistent_vars import read_var
from libsan.host.linux import mkdir, mount, umount
from libsan.host.cmdline import run
import time


def stopped_pool_preserve_data():
    errors = []

    stratis = Stratis()
    pool_name = read_var("STRATIS_POOL")
    pool_uuid = read_var("STRATIS_POOL_UUID")
    fs_name = "fs_test"
    file_name = "test_file"
    mount_dir = "/mnt/pool_preserve_data"
    message = "this is pool preserve data test"
    stratis_fs_path = f"/dev/stratis/{pool_name}/{fs_name}"

    atomic_run(f"Creating filesystem: {fs_name}",
               pool_name=pool_name,
               fs_name=fs_name,
               command=stratis.fs_create,
               errors=errors
               )

    time.sleep(2)

    atomic_run(f"Creating dir: {mount_dir}",
               new_dir=mount_dir,
               command=mkdir,
               errors=errors
               )

    atomic_run(f"Mounting filesystem: {fs_name}",
               device=stratis_fs_path,
               mountpoint=mount_dir,
               command=mount,
               errors=errors
               )

    atomic_run(f"Writing data to file: {fs_name}",
               cmd=f"echo '{message}' > {mount_dir}/{file_name}",
               command=run,
               errors=errors
               )

    atomic_run("Writing 1GB of random data to stratis fs",
               cmd=f"dd if=/dev/random of={mount_dir}/random_data bs=1M count=1000",
               command=run,
               errors=errors
               )

    atomic_run(f"Trying to stop pool with mounted fs, this should fail: {fs_name}",
               pool_name=pool_name,
               command=stratis.pool_stop,
               expected_ret=1,
               expected_out=["stratis_cli._errors.StratisCliEngineError: ERROR: low-level ioctl error due to nix error;"
                             " header result:", "error: EBUSY: Device or resource busy"],
               errors=errors
               )

    atomic_run(f"Unmounting fs: {fs_name}",
               mountpoint=mount_dir,
               command=umount,
               errors=errors
               )

    atomic_run("Listing stratis filesystems",
               command=stratis.fs_list,
               errors=errors
               )

    atomic_run("Trying to stop pool with unmounted fs",
               pool_name=pool_name,
               command=stratis.pool_stop,
               errors=errors
               )

    atomic_run("Starting pool to verify data",
               pool_uuid=pool_uuid,
               command=stratis.pool_start,
               errors=errors
               )

    atomic_run("Listing stratis filesystems",
               command=stratis.fs_list,
               errors=errors
               )

    atomic_run(f"Mounting filesystem: {fs_name}",
               device=stratis_fs_path,
               mountpoint=mount_dir,
               command=mount,
               errors=errors
               )

    ret, data = atomic_run("Check data on filesystem",
                           return_output=True,
                           cmd=f"cat {mount_dir}/{file_name}",
                           command=run,
                           errors=errors
                           )

    if message not in data:
        errors.append(f"FAIL: Could not find message: {message} in returned output!")

    atomic_run("Writing 1GB of random data to stratis fs to verify fs is still usable",
               cmd=f"dd if=/dev/random of={mount_dir}/random_data2 bs=1M count=1000",
               command=run,
               errors=errors
               )

    atomic_run(f"Unmounting fs: {fs_name}",
               mountpoint=mount_dir,
               command=umount,
               errors=errors
               )

    atomic_run(f"Destroying filesystem: {fs_name}",
               pool_name=pool_name,
               fs_name=fs_name,
               command=stratis.fs_destroy,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = stopped_pool_preserve_data()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.lio import TargetCLI
from libsan.host.cmdline import run
from time import sleep
import os


def count_files(directory):
    if not os.path.isdir(directory):
        return None
    number = len(os.listdir(directory))
    return number


def saveconfig_test_compare():
    errors = []
    target = TargetCLI()
    max_files = 0

    ret, data = atomic_run("Getting parameter max_backup_files from group global",
                           return_output=True,
                           command=target.get,
                           group="global",
                           max_backup_files="max_backup_files",
                           errors=errors
                           )
    if ret == 0:
        max_files = int(data.split("=")[1])

    ret = count_files("/etc/target/backup/")

    if not ret:
        print("FAIL: Backup directory does not exist.")
        errors.append("FAIL: Backup directory does not exist.")
    elif ret > max_files:
        print("FAIL: In backup directory is more files than max_backup_file!")
        errors.append("FAIL: In backup directory is more files than max_backup_file!")

    return errors


def saveconfig_create():
    errors = []
    target = TargetCLI()
    backups = int(os.environ["fmf_number_of_backups"])

    for i in range(backups):
        atomic_run("Creating saveconfig.json number %s" % i,
                   command=target.saveconfig,
                   errors=errors
                   )

    number = count_files("/etc/target/backup/")

    if number > 1:
        print("FAIL: In '/etc/target/backup/' is more than 1 backup file.")
        errors.append("FAIL: In '/etc/target/backup/' is more than 1 backup file.")
        return errors

    saveconfig_test_compare()

    return errors


def saveconfig_fail():
    errors = []
    target = TargetCLI()
    ret, data = atomic_run("Getting parameter max_backup_files from group global",
                           return_output=True,
                           command=target.get,
                           group="global",
                           max_backup_files="max_backup_files",
                           errors=errors
                           )

    maximum = int(data.split("=")[1])
    for i in range(maximum + 2):
        target.path = "/backstores/fileio"

        atomic_run("Creating fileio object number: %s." % i,
                   name="saveconfig%s" % i,
                   file_or_dev="/var/tmp/saveconfig_file%s" % i,
                   size="1",
                   command=target.create,
                   errors=errors
                   )

        target.path = ""

        atomic_run("Creating saveconfig.json number %s" % i,
                   command=target.saveconfig,
                   errors=errors
                   )
        sleep(1)

    ret = count_files("/etc/target/backup/")

    if not ret:
        print("FAIL: Backup directory does not exist.")
        errors.append("FAIL: Backup directory does not exist.")
    elif maximum != ret:
        print("FAIL: In backup directory is more files than max_backup_file!")
        errors.append("FAIL: In backup directory is more files than max_backup_file!")

    target.path = "/backstores/fileio"

    for i in range(maximum + 2):
        atomic_run("Removing fileio object number: %s" % i,
                   name="saveconfig%s" % i,
                   command=target.delete,
                   errors=errors
                   )

        atomic_run("Removing saveconfig file",
                   cmd="rm -f /var/tmp/saveconfig_file%s" % i,
                   command=run,
                   errors=errors
                   )
    return errors


def saveconfig_cleanup():
    errors = []

    atomic_run("Removing backups from /etc/target/backup",
               cmd="rm -rf /etc/target/backup/*",
               command=run,
               errors=errors
               )

    atomic_run("Removing saveconfig.json",
               cmd="rm -rf /etc/target/saveconfig.json",
               command=run,
               errors=errors
               )
    if errors:
        print("Cleanup failed")
        errors.append("Cleanup failed")

    return errors


if __name__ == "__main__":
    if int(os.environ['fmf_tier']) == 1:
        errs = saveconfig_create()
        errs += saveconfig_cleanup()
    if int(os.environ['fmf_tier']) == 2:
        errs = saveconfig_fail()
        errs += saveconfig_cleanup()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""tc.py: Module to help on test case execution."""

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2015 Red Hat, Inc. All rights reserved."

import re  # regex
import sys
import os
import libsan.host.linux as linux
import stqe.host.LogChecker
import stqe.host.beaker
import libsan.host.mp as mp
from libsan.host.cmdline import run


def show_sys_info():
    print("### Kernel Info: ###")
    ret, kernel = run("uname -a", return_output=True, verbose=False)
    ret, taint_val = run("cat /proc/sys/kernel/tainted", return_output=True, verbose=False)
    print("Kernel version: %s" % kernel)
    print("Kernel tainted: %s" % taint_val)
    print("### IP settings: ###")
    run("ip a")
    print("### File system disk space usage: ###")
    run("df -h")

    if run("rpm -q device-mapper-multipath") == 0 and linux.service_status("multipathd") == 0:
        # Abort test execution if multipath is not working well
        if run("multipath -l 2>/dev/null") != 0:
            sys.exit(1)
        # Flush all unused multipath devices before starting the test
        mp.flush_all()
        mp.multipath_reload()


class TestClass:
    # we currently support these exit code for a test case
    tc_sup_status = {"pass": "PASS: ",
                     "fail": "ERROR: ",
                     "skip": "SKIP: "}
    initial_loaded_modules = []
    final_loaded_modules = []

    test_dir = "%s/.stqe-test" % os.path.expanduser("~")
    test_log = "%s/test.log" % test_dir

    def __init__(self):
        self.tc_pass = []
        self.tc_fail = []
        self.tc_skip = []  # For some reason it did not execute
        self.tc_results = []  # Test results stored in a list
        f = None
        print("################################## Test Init ###################################")
        stqe.host.LogChecker.check_all()
        if not os.path.isdir(self.test_dir):
            linux.mkdir(self.test_dir)
        # read entries on test.log, there will be entries if tend was not called
        # before starting a TC class again, usually if the test case reboots the server
        if not os.path.isfile(self.test_log):
            # running the test for the first time
            show_sys_info()
            # Track memory usage during test
            run("free -b > init_mem.txt", verbose=False)
            run("top -b -n 1 > init_top.txt", verbose=False)
        else:
            try:
                f = open(self.test_log)
                file_data = f.read()
                f.close()
            except Exception as e:
                print(e)
                print("FAIL: TestClass() could not read %s" % self.test_log)
                return
            finally:
                f.close()
            log_entries = file_data.split("\n")
            # remove the file, once tlog is ran it will add the entries again...
            run("rm -f %s" % self.test_log, verbose=False)
            if log_entries:
                print("INFO: Loading test result from previous test run...")
                for entry in log_entries:
                    self.tlog(entry)
        print("################################################################################")
        self.initial_loaded_modules = linux.get_all_loaded_modules()
        return

    def tlog(self, string):
        """print message, if message begins with supported message status
        the test message will be added to specific test result array
        """
        print(string)
        if re.match(self.tc_sup_status["pass"], string):
            self.tc_pass.append(string)
            self.tc_results.append(string)
            run("echo '%s' >> %s" % (string, self.test_log), verbose=False)
        if re.match(self.tc_sup_status["fail"], string):
            self.tc_fail.append(string)
            self.tc_results.append(string)
            run("echo '%s' >> %s" % (string, self.test_log), verbose=False)
        if re.match(self.tc_sup_status["skip"], string):
            self.tc_skip.append(string)
            self.tc_results.append(string)
            run("echo '%s' >> %s" % (string, self.test_log), verbose=False)
        return None

    @staticmethod
    def trun(cmd, return_output=False):
        """Run the cmd and format the log. return the exitint status of cmd
        The arguments are:
        \tCommand to run
        \treturn_output: if should return command output as well (Boolean)
        Returns:
        \tint: Command exit code
        \tstr: command output (optional)
        """
        return run(cmd, return_output)

    def tok(self, cmd, return_output=False):
        """Run the cmd and expect it to pass.
        The arguments are:
        \tCommand to run
        \treturn_output: if should return command output as well (Boolean)
        Returns:
        \tBoolean: return_code
        \t\tTrue: If command excuted successfully
        \t\tFalse: Something went wrong
        \tstr: command output (optional)
        """
        output = None
        if not return_output:
            cmd_code = run(cmd)
        else:
            cmd_code, output = run(cmd, return_output)

        if cmd_code == 0:
            self.tpass(cmd)
            ret_code = True
        else:
            self.tfail(cmd)
            ret_code = False

        if return_output:
            return ret_code, output
        else:
            return ret_code

    def tnok(self, cmd, return_output=False):
        """Run the cmd and expect it to fail.
        The arguments are:
        \tCommand to run
        \treturn_output: if should return command output as well (Boolean)
        Returns:
        \tBoolean: return_code
        \t\tFalse: If command excuted successfully
        \t\tTrue: Something went wrong
        \tstr: command output (optional)
        """
        output = None
        if not return_output:
            cmd_code = run(cmd)
        else:
            cmd_code, output = run(cmd, return_output)

        if cmd_code != 0:
            self.tpass(cmd + " [exited with error, as expected]")
            ret_code = True
        else:
            self.tfail(cmd + " [expected to fail, but it did not]")
            ret_code = False

        if return_output:
            return ret_code, output
        else:
            return ret_code

    def tpass(self, string):
        """Will add PASS + string to test log summary
        """
        self.tlog(self.tc_sup_status["pass"] + string)
        return None

    def tfail(self, string):
        """Will add ERROR + string to test log summary
        """
        self.tlog(self.tc_sup_status["fail"] + string)
        return None

    def tskip(self, string):
        """Will add SKIP + string to test log summary
        """
        self.tlog(self.tc_sup_status["skip"] + string)
        return None

    def tend(self):
        """It checks for error in the system and print test summary
        Returns:
        \tBoolean
        \t\tTrue if all test passed and no error was found on server
        \t\tFalse if some test failed or found error on server
        """
        if stqe.host.LogChecker.check_all():
            self.tpass("Search for error on the server")
        else:
            self.tfail("Search for error on the server")

        self.final_loaded_modules = linux.get_all_loaded_modules()
        if self.final_loaded_modules:
            for module in self.final_loaded_modules:
                if module not in self.initial_loaded_modules:
                    # in case the module was unloaded already, like via dependency...
                    if not linux.is_module_loaded(module):
                        continue
                    self.tlog("module '%s' was loaded during the test. Unloading it..." % module)
                    if not linux.unload_module(module, remove_dependent=True):
                        # some modules like ext4, seems load some module dependencies that
                        # make very hard to unload it again.
                        self.tlog("INFO: Couldn't unload module '%s', ignoring it..." % module)

        print("################################ Test Summary ##################################")
        # for tc in self.tc_pass:
        #    print tc
        # for tc in self.tc_fail:
        #    print tc
        # for tc in self.tc_skip:
        #    print tc
        # Will print test results in order and not by test result order
        for tc in self.tc_results:
            print(tc)

        n_tc_pass = len(self.tc_pass)
        n_tc_fail = len(self.tc_fail)
        n_tc_skip = len(self.tc_skip)
        print("#############################")
        print("Total tests that passed: " + str(n_tc_pass))
        print("Total tests that failed: " + str(n_tc_fail))
        print("Total tests that skipped: " + str(n_tc_skip))
        print("################################################################################")
        sys.stdout.flush()
        # Added this sleep otherwise some of the prints were not being shown....
        linux.sleep(1)
        run("rm -f %s" % self.test_log, verbose=False)
        run("rmdir %s" % self.test_dir, verbose=False)

        # If at least one test failed, return error
        if n_tc_fail > 0:
            return False

        return True

    def log_submit(self, log_file):
        """It will upload logs depending on environment used to run the test
        """
        if not log_file:
            self.tfail("log_submit: log_file parameter not set")
            return False

        if stqe.host.beaker.is_bkr_job():
            stqe.host.beaker.log_submit(log_file)

        self.tlog("doesn't know how to upload log in this environment. Skipping...")
        return True
